subroutine BoundValGradIncr(Initialize)
! Purpose: To increase the value of boundary values gradually with 
!          time.

use CommonVars, only: Prec,BoundSide,EquationType,StartBoundNode,    &
   EndBoundNode,IncreaseAmount,IncreaseTime,PresBoundVals,           &
   TempBoundVals,SaliBoundVals,LogFile,StartBoundVals,NumNodesX,     &
   NumNodesY,NumNodesZ,Time
implicit none
real (selected_real_kind(Prec)) :: Slope,StartVal,EndVal,CurrVal
integer :: I,J,K,Initialize

if (Initialize == 1) then
!---------------------------------------------------------------------
if (StartBoundNode > EndBoundNode) then
   write(LogFile,*) 'StartBoundNode must be less than or equal to'
   write(LogFile,*) 'EndBoundNode -BoundValGradIncr'
   write(LogFile,*) 'Terminating'
   stop
else
   J = 1
   if (BoundSide == 'T') then
      K = 1
   elseif (BoundSide == 'B') then
      K = NumNodesZ
   elseif (BoundSide == 'W') then
      I = 1
   elseif (BoundSide == 'E') then
      I = NumNodesX
   else
      write(LogFile,*) 'BoundSide not specified -BoundValGradIncr'
      write(LogFile,*) 'Terminating'
      stop
   end if

   if ((BoundSide == 'T').or.(BoundSide == 'B')) then
      do I=StartBoundNode,EndBoundNode
         if (EquationType=='P') then
            StartBoundVals(I) = PresBoundVals(I,J,K)
         elseif (EquationType=='X') then
            StartBoundVals(I) = SaliBoundVals(I,J,K)
         elseif (EquationType=='T') then
            StartBoundVals(I) = TempBoundVals(I,J,K)
         else
            write(LogFile,*) 'Equation type not specified -BoundValGradIncr'
            write(Logfile,*) 'Terminating'
            stop
         end if
      end do
   elseif ((BoundSide == 'W').or.(BoundSide == 'E')) then
      do K=StartBoundNode,EndBoundNode
         if (EquationType=='P') then
            StartBoundVals(K) = PresBoundVals(I,J,K)
         elseif (EquationType=='X') then
            StartBoundVals(K) = SaliBoundVals(I,J,K)
         elseif (EquationType=='T') then
            StartBoundVals(K) = TempBoundVals(I,J,K)
         else
            write(LogFile,*) 'Equation type not specified -BoundValGradIncr'
            write(Logfile,*) 'Terminating'
            stop
         end if
      end do
   end if
end if
!---------------------------------------------------------------------
elseif (Initialize == 0) then
   J = 1
   if (BoundSide == 'T') then
      K = 1
   elseif (BoundSide == 'B') then
      K = NumNodesZ
   elseif (BoundSide == 'W') then
      I = 1
   elseif (BoundSide == 'E') then
      I = NumNodesX
   end if

   if ((BoundSide == 'T').or.(BoundSide == 'B')) then
      do I=StartBoundNode,EndBoundNode
         if (Time <= IncreaseTime) then
            StartVal = StartBoundVals(I)
            EndVal   = StartVal + IncreaseAmount
            Slope    = (EndVal-StartVal)/IncreaseTime
            CurrVal  = Slope*Time + StartVal
            if (EquationType == 'P') then
               PresBoundVals(I,J,K) = CurrVal
            elseif (EquationType == 'X') then
               SaliBoundVals(I,J,K) = CurrVal
            elseif (EquationType == 'T') then
               TempBoundVals(I,J,K) = CurrVal
            end if
         end if
      end do
   elseif ((BoundSide == 'W').or.(BoundSide == 'E')) then
      do K=StartBoundNode,EndBoundNode
         if (Time <= IncreaseTime) then
            StartVal = StartBoundVals(K)
            EndVal   = StartVal + IncreaseAmount
            Slope    = (EndVal-StartVal)/IncreaseTime
            CurrVal  = Slope*Time + StartVal
            if (EquationType == 'P') then
               PresBoundVals(I,J,K) = CurrVal
            elseif (EquationType == 'X') then
               SaliBoundVals(I,J,K) = CurrVal
            elseif (EquationType == 'T') then
               TempBoundVals(I,J,K) = CurrVal
            end if
         end if
      end do
   end if
end if

end
