recursive function CalcBulkDens(Tempr,Presr,Salin) Result(Answer)
! note: returns -1 if there is an error

use CommonVars, only: DensSlices,LogFile,Prec,HaliteDens, &
   CompressFactor,MinPres
implicit none
real(selected_real_kind(Prec)) :: Answer,S2PhaseL,S2PhaseV,LiqSal, &
   VapSal,Tempr,LiqMassFrac,VapMassFrac,LiqDens,VapDens,           &
   LiqVolFrac,VapVolFrac,Presr,Salin,Interp3D,D2PhaseL,D2PhaseV,   &
   T11,T12,T21,T22,SlopeT1,SlopeT2,T1,T2,S1,S2,Ans1,Ans2,Slope,    &
   Pres3Phase,Pres3Ph,S3PhaseL,CritPres,CritiPres,PresFrac,        &
   S3PhaseV,SolubilityV, SaliTol,S2PhL,S2PhV,Den3PhL,Den3PhV,      &
   Sal3PhL,HaliteSat,Term1,Term2,Sal3PhV,RefPres,Alpha,HalDens,    &
   HalRefDens,Beta,RefTemp
integer :: PhaseIndex,PhaseInd

CritPres = CritiPres(Tempr)
T11 = 280.D0
T12 = 200.D0
T21 = 320.D0
T22 = 350.D0
S1 = 0.D0
S2 = 1.D0
SlopeT1 = (T12-T11)/(S2-S1)
SlopeT2 = (T22-T21)/(S2-S1)
T1 = SlopeT1*(Salin-S1)+T11
T2 = SlopeT2*(Salin-S1)+T21
PhaseInd = PhaseIndex(Tempr,Presr,Salin)
if (PhaseInd.ne.-100) then
!---------------------------------------------------------------------
Pres3Ph = Pres3Phase(Tempr)
if ((Tempr>T1).and.(Tempr<T2)) then
   Ans1 = CalcBulkDens(T1,Presr,Salin)
   Ans2 = CalcBulkDens(T2,Presr,Salin)
   if ((Ans1==-1.D0).or.(Ans2==-1.D0)) then
      write(LogFile,*) 'Problem computing bulk density - CalcBulkDens'
      write(LogFile,*) 'Halting'
      stop
   end if
   Slope = (Ans2-Ans1)/(T2-T1)
   Answer = Slope*(Tempr-T1)+Ans1
elseif (((Presr<=CritPres).and.(Presr>=Pres3Ph)).and. &
       ((PhaseInd.ne.0).and.(PhaseInd.ne.-2))) then
   ! Note: the bulk densities and the upper two-phase boundary densities
   ! from Anderko and Pitzer match up well near the critical pressure,
   ! but the match becomes worse as the pressure gets nearer to the
   ! 3-phase coexistence pressure. The lines below interpolate between
   ! the two within a certain range of salinities near the two-phase 
   ! boundary, where the range of salinities depends on how near the 
   ! pressure is to the critical pressure.
   S2PhL = S2PhaseL(Tempr,Presr)
   S2PhV = S2PhaseV(Tempr,Presr)
   SaliTol = 1.D0
   PresFrac = Presr/CritPres
   SaliTol = SaliTol*(1.D0-PresFrac)
   if (Salin>S2PhL) then
      SaliTol = MIN(SaliTol,S3PhaseL(Tempr)-S2PhL)
   else
      SaliTol = MIN(SaliTol,S2PhV-S3PhaseV(Tempr))
   end if
   if (SaliTol==0.D0) then
      Answer = D2PhaseL(Tempr,Presr)
   elseif (DABS(Salin-S2PhL)<SaliTol) then
      S1 = S2PhL
      S2 = S1 + SaliTol
      Ans1 = D2PhaseL(Tempr,Presr)
      Ans2 = Interp3D(Tempr,Presr,S2,DensSlices)
      Slope = (Ans2-Ans1)/(S2-S1)
      Answer = Slope*(Salin-S1) + Ans1
   elseif (DABS(Salin-S2PhV)<SaliTol) then
      S1 = S2PhV
      S2 = S1 - SaliTol
      Ans1 = D2PhaseV(Tempr,Presr)
      Ans2 = Interp3D(Tempr,Presr,S2,DensSlices)
      Slope = (Ans2-Ans1)/(S2-S1)
      Answer = Slope*(Salin-S1) + Ans1
   else
      Answer = Interp3D(Tempr,Presr,Salin,DensSlices)
   end if
elseif (PhaseInd==0) then
   LiqSal = S2PhaseL(Tempr,Presr)
   VapSal = S2PhaseV(Tempr,Presr)
   if (DABS(LiqSal-VapSal)>0.D0) then
      LiqMassFrac = (Salin-VapSal)/(LiqSal-VapSal)
   else
      LiqMassFrac = 0.D0
   end if
   VapMassFrac = 1.D0-LiqMassFrac
   LiqDens = D2PhaseL(Tempr,Presr)
   VapDens = D2PhaseV(Tempr,Presr)
   LiqVolFrac=VapDens*LiqMassFrac/   &
      (VapDens*LiqMassFrac + LiqDens*VapMassFrac)
   VapVolFrac=1.D0-LiqVolFrac
   Answer=LiqVolFrac*LiqDens+VapVolFrac*VapDens
elseif (PhaseInd==2) then
   Alpha = 1.D-4
   Beta  = 1.D-1
   RefPres = Pres3Phase(Tempr)
   RefTemp = 0.D0
   HalRefDens = HaliteDens
   HalDens = HalRefDens+(Alpha*(Presr-RefPres)-Beta*(Tempr-RefTemp))
   Sal3PhV = SolubilityV(Tempr,Presr)
   Den3PhV = Interp3D(Tempr,Presr,Sal3PhV,DensSlices)
   Term1 = Den3PhV*(Salin-Sal3PhV)
   Term2 = HalDens*(1.D0-Salin)
   HaliteSat = Term1/(Term1+Term2)
   VapVolFrac = 1.D0-HaliteSat
   Answer = VapVolFrac*Den3PhV + HaliteSat*HalDens
elseif (PhaseInd==-2) then
   Alpha = 1.D-4
   Beta  = 1.D-1
   RefPres = Pres3Phase(Tempr)
   RefTemp = 0.D0
   HalRefDens = HaliteDens
   HalDens = HalRefDens+(Alpha*(Presr-RefPres)-Beta*(Tempr-RefTemp))
   Sal3PhL = S3PhaseL(Tempr)
   Den3PhL = Interp3D(Tempr,Presr,Sal3PhL,DensSlices)
   Term1 = Den3PhL*(Salin-Sal3PhL)
   Term2 = HalDens*(1.D0-Salin)
   HaliteSat = Term1/(Term1+Term2)
   LiqVolFrac = 1.D0-HaliteSat
   Answer = LiqVolFrac*Den3PhL + HaliteSat*HalDens 
else
   Answer = Interp3D(Tempr,Presr,Salin,DensSlices)
end if
RefPres = MinPres
Answer = Answer + CompressFactor*(Presr-RefPres)
! The line below is only for the Elder problem simulation!!
! See also the lines at the end of ThermProps and in 
! GetVelocities
!Answer = 1000.D0 + 200.D0*Salin
!---------------------------------------------------------------------
else
   Answer = -1.D0
end if
end
