function CritiSali(Tempr)
! Input: temperature (C)
! Output: salinity (wt.frac.NaCl) on the critical curve.

! The critical salinities are the salinities on the 
! upper boundary of the two-phase surface at the
! critical pressure (see CritiPres and Base2Phase).

use CommonVars, only: TempArray,LiqSali2Ph,Prec
implicit none
real(selected_real_kind(Prec)) :: CritiSali,Tempr,T1,T2,S1,S2,Slope
integer :: Row1,Row2

if (Tempr>=300.D0) then
   call FindTemp(Tempr,Row1,Row2)
   T1 = TempArray(Row1)
   T2 = TempArray(Row2)
   S1 = LiqSali2Ph(Row1-30,1)
   S2 = LiqSali2Ph(Row2-30,1)
   Slope = (S2-S1)/(T2-T1)
   CritiSali = (Slope*(Tempr-T1)+S1)/1.D2
else
   CritiSali = 0.D0
end if
end
