subroutine DensDerivs
! Purpose: To calculate the density derivatives used in the 
!          pressure equation - DRhoDPres,DRhoDSali, and  
!          DRhoDTemp, where Rho = bulk density

use CommonVars, only: Prec,DRhoDPres,DRhoDTemp,DRhoDSali,NumNodesX,  &
   NumNodesY,NumNodesZ,Temp,Pres,Sali,OldTemp,OldPres,OldSali,       &
   DeltaMassGlobal,Delta_t
implicit none
real (selected_real_kind(Prec)) :: CalcDRhoDPres,CalcDRhoDTemp,      &
   CalcDRhoDSali,Tempr,Presr,Salin,OldTempr,OldSalin,OldPresr,       &
   DRhoDPres1,DRhoDPres2,DRhoDTemp1,DRhoDTemp2,DRhoDSali1,           &
   DRhoDSali2,DSaliDt,DTempDt
integer :: I,J,K,Mode

do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         Tempr = Temp(I,J,K)
         Salin = Sali(I,J,K)
         Presr = Pres(I,J,K)
         
         OldTempr = OldTemp(I,J,K)
         OldSalin = OldSali(I,J,K)
         OldPresr = OldPres(I,J,K)

         DSaliDt = (Salin-OldSalin)/Delta_t
         DTempDt = (Tempr-OldTempr)/Delta_t

         DRhoDPres1 = CalcDRhoDPres(Tempr,Presr,Salin)
         DRhoDPres2 = CalcDRhoDPres(OldTempr,OldPresr,OldSalin)
         if (DeltaMassGlobal > 0.D0) then
            if (DRhoDPres1>DRhoDPres2) then
               DRhoDPres(I,J,K) = DRhoDPres1
            else
               DRhoDPres(I,J,K) = DRhoDPres2
            end if
         else
            if (DRhoDPres1<DRhoDPres2) then
               DRhoDPres(I,J,K) = DRhoDPres1
            else
               DRhoDPres(I,J,K) = DRhoDPres2
            end if
         end if

         DRhoDTemp1 = CalcDRhoDTemp(Tempr,Presr,Salin)
         DRhoDTemp2 = CalcDRhoDTemp(OldTempr,OldPresr,OldSalin)
         if (DeltaMassGlobal < 0.D0) then
            if (DRhoDTemp1*DTempDt>DRhoDTemp2*DTempDt) then
               DRhoDTemp(I,J,K) = DRhoDTemp1
            else
               DRhoDTemp(I,J,K) = DRhoDTemp2
            end if
         else
            if (DRhoDTemp1*DTempDt<DRhoDTemp2*DTempDt) then
               DRhoDTemp(I,J,K) = DRhoDTemp1
            else
               DRhoDTemp(I,J,K) = DRhoDTemp2
            end if
         end if

         DRhoDSali1 = CalcDRhoDSali(Tempr,Presr,Salin)
         DRhoDSali2 = CalcDRhoDSali(OldTempr,OldPresr,OldSalin)
         if (DeltaMassGlobal < 0.D0) then
            if (DRhoDSali1*DSaliDt>DRhoDSali2*DSaliDt) then
               DRhoDSali(I,J,K) = DRhoDSali1
            else
               DRhoDSali(I,J,K) = DRhoDSali2
            end if
         else
            if (DRhoDSali1*DSaliDt<DRhoDSali2*DSaliDt) then
               DRhoDSali(I,J,K) = DRhoDSali1
            else
               DRhoDSali(I,J,K) = DRhoDSali2
            end if
         end if
      end do
   end do
end do
end 
