FUNCTION FViscosity(Tempr,Presr,Salin)
! Purpose: To calculate the the dynamic viscosity (Pa*s) of 
!          the fluid (liquid and vapor).
! Input: Temperature (C), pressure (Pa), salinity (wt.
!        fraction NaCl.
! Output: Fluid viscosity (Pa*s)

use CommonVars, only: Prec,LogFile
implicit none
real (selected_real_kind(Prec)) :: Tempr,Presr,Salin,             &
     PureWaterDens,PureWaterVisc,SVisc,T,FViscosity,A0,A1,        &
     PWViscosity,CalcBulkDens,PWSali
data A0/4.71624D-3/, A1/-4.0203D-6/, PWSali/0.D0/

PureWaterDens = CalcBulkDens(Tempr,Presr,PWSali)
if (PureWaterDens==-1.D0) then
   write(LogFile,*) 'CalcBulkDens returned an error -FViscosity'
   write(LogFile,*) 'Halting'
   stop
end if
PureWaterVisc = PWViscosity(Tempr,PureWaterDens)
if (Tempr<800.D0) then
   SVisc = A0 + A1*Tempr
   T = Tempr/800.D0
   FViscosity = ( PureWaterVisc * (1.D0+3.D0*Salin) * (1.D0-T)**9 &
                + T**9 * (PureWaterVisc*(1.D0-Salin)+             &
                SVisc*Salin) ) / ( (1.D0-T)**9+T**9 )
else
   if (Tempr<1000.D0) then
     SVisc = A0 + A1*Tempr
   else
     SVisc = A0 + A1*1000.D0
   endif
   FViscosity = PureWaterVisc * (1.D0-Salin) + SVisc * Salin
endif
! The line below is only for the Elder problem!
!FViscosity = 0.001D0 
end
