subroutine FindPres2Ph(presr,refrow,c1,c2)
! Purpose: to locate the pressure within PresMat2Ph as part 
!          of the procedure to interpolate between values 
!          of density and salinity in the 2D lookup tables 
!          for the two-phase region boundary.
! Input: pressure (Pa) and a reference row
! Output: column indices c1 and c2 of PresMat2Ph 
!    such that PresMat2Ph(RefRow,c1)<=Presr/1.D5<= 
!    PresMat2Ph(RefRow,c2)

use CommonVars, only: Prec,PresMat2Ph
real (selected_real_kind(Prec)) :: Presr, Avg, Pi, Pf, Pm
integer :: c1, c2, ci, cf, cm, Flag, RefRow
! Strategy: divide the matrix in two halves, and find 
! which half Presr belongs in.  Keep doing this until
! it is not possible anymore.

! Reminder: There are 75 columns in presmat

Presr = Presr/1.D5
Flag = 0
ci = 1
cf = 75
Avg = (ci+cf)/2
cm = IDNINT(avg)
Pi = PresMat2Ph(RefRow,ci)
Pf = PresMat2Ph(RefRow,cf)
Pm = PresMat2Ph(RefRow,cm)
do while (Flag.NE.1)
   if ((Presr<=Pi).AND.(Presr>=Pm)) then
      cf = cm
      Avg = (ci+cf)/2
      cm = IDNINT(avg)
      Pf = PresMat2Ph(RefRow,cf)
      Pm = PresMat2Ph(RefRow,cm)
      if ((cm.EQ.ci).OR.(cm.EQ.cf)) then
         Flag = 1
      end if
   else
      ci = cm
      Avg = (ci+cf)/2
      cm = IDNINT(avg)
      Pi = PresMat2Ph(RefRow,ci)
      Pm = PresMat2Ph(RefRow,cm)
      if ((cm.EQ.ci).OR.(cm.EQ.cf)) then
         Flag = 1
      end if
   end if
end do
Presr = Presr*1.D5
c1 = ci
c2 = cf
end
