subroutine FindTemp(Tempr,r1,r2)
! Purpose: given a temperature (C), returns the row indices 
!          r1 and r2 of TempMat such that 
!          TempMat(r1,1) <= Tempr <= TempMat(r2,1)
! This procedure is used by both the bulk quantity and the 
! two-phase boundary interpolation schemes. The two-phase
! boundary values are not needed below 300 C, and so there
! are 30 less of those values than there are of bulk 
! quantities; hence, the results must be shifted by 30 rows 
! when this procedure is used to get two-phase boundary 
! values.

! Note: the algorithm employed below is the bisection 
! procedure; bisection is used in all the search procedures
! in FISHES. This procedure is described in just about any
! book on numerical analysis - see, for example, Atkinson
! (1989) (reference below).

use CommonVars, only: Prec,TempArray
real (selected_real_kind(Prec)) :: Tempr, Avg, Ti, Tf, Tm
integer :: r1, r2, ri, rf, rm, Flag
! Strategy: divide the matrix in two halves, and find 
! which half Tempr belongs in.  Keep doing this until
! it is not possible anymore.

! Reminder: There are 79 rows in TempMat
Flag = 0
ri = 1
rf = 79
Avg = (ri+rf)/2
rm = IDNINT(avg)
Ti = TempArray(ri)
Tf = TempArray(rf)
Tm = TempArray(rm)
do while (Flag.NE.1)
   if ((Tempr.GE.Ti).AND.(Tempr.LE.Tm)) then
      rf = rm
      Avg = (ri+rf)/2
      rm = IDNINT(avg)
      Tf = TempArray(rf)
      Tm = TempArray(rm)
      if ((rm.EQ.ri).OR.(rm.EQ.rf)) then
         Flag = 1
      end if
   else
      ri = rm
      Avg = (ri+rf)/2
      rm = IDNINT(avg)
      Ti = TempArray(ri)
      Tm = TempArray(rm)
      if ((rm.EQ.ri).OR.(rm.EQ.rf)) then
         flag = 1
      end if
   end if
end do
r1 = ri
r2 = rf
end

! Atkinson KE, (1989) an introduction to numerical analysis, 
!     2nd ed, John Wiley & Sons, pg. 56.
