subroutine GetPresCoeffs
! Purpose: To calculate the coefficients for the pressure equation.

use CommonVars, only:Prec,PresGamma1X,PresGamma1Y,PresGamma1Z, &
    NumNodesX,NumNodesY,NumNodesZ,PresNodeTypes,PresCoeffs,    &
    XNodeWidths,YNodeWidths,ZNodeWidths,Pres,PresBoundVals,    &
    Porosities,DRhoDPres,Delta_t,OldPres,PhaseIndices
implicit none
real (selected_real_kind(Prec)) :: NodeDistance,aN,aS,aW,aE,aT,aB, &
     QP_Avg,Source
integer :: I,J,K

do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         if ((PresNodeTypes(I,J,K)=='I').or. &
             (PresNodeTypes(I,J,K)=='F')) then
            !--------------------------------------------------------
            !                     coeff: aN
            !--------------------------------------------------------
            aN=0.D0
            PresCoeffs(I,J,K,2)=aN
            !--------------------------------------------------------
            !                     coeff: aS
            !--------------------------------------------------------
            aS=0.D0
            PresCoeffs(I,J,K,3)=aS
            !--------------------------------------------------------
            !                     coeff: aW
            !--------------------------------------------------------
            if (I>1) then ! there is a node to the west
               NodeDistance = XNodeWidths(I-1)/2.D0             &
                              + XNodeWidths(I)/2.D0
               aW=PresGamma1X(I-1,J,K)/(NodeDistance*XNodeWidths(I))
            else
               aW=0.D0
            end if
            PresCoeffs(I,J,K,4)=aW
            !--------------------------------------------------------
            !                     coeff: aE
            !--------------------------------------------------------
            if (I<NumNodesX) then ! there is a node to the east
               NodeDistance = XNodeWidths(I+1)/2.D0             &
                              + XNodeWidths(I)/2.D0
               aE=PresGamma1X(I,J,K)/(NodeDistance*XNodeWidths(I))
            else
               aE=0.D0
            end if
            PresCoeffs(I,J,K,5)=aE
            !--------------------------------------------------------
            !                     coeff: aB
            !--------------------------------------------------------
            if (K<NumNodesZ) then ! there is a node below
               NodeDistance = ZNodeWidths(K+1)/2.D0             &
                              + ZNodeWidths(K)/2.D0
               aB=PresGamma1Z(I,J,K)/(NodeDistance*ZNodeWidths(K))
            else
               aB=0.D0
            end if
            PresCoeffs(I,J,K,6)=aB
            !--------------------------------------------------------
            !                     coeff: aT
            !--------------------------------------------------------
            if (K>1) then ! there is a node above
               NodeDistance = ZNodeWidths(K-1)/2.D0             &
                              + ZNodeWidths(K)/2.D0
               aT=PresGamma1Z(I,J,K-1)/(NodeDistance*ZNodeWidths(K))
            else
               aT=0.D0
            end if
            PresCoeffs(I,J,K,7)=aT
            !--------------------------------------------------------
            !                     coeff: aP
            !--------------------------------------------------------
            PresCoeffs(I,J,K,1)=aN+aS+aW+aE+aB+aT+ &
               Porosities(I,J,K)*DRhoDPres(I,J,K)/Delta_t
            !--------------------------------------------------------
            !                     source term
            !--------------------------------------------------------
            call GetQP_Avg(I,J,K,QP_Avg)
            Source = QP_Avg + OldPres(I,J,K)* &
               Porosities(I,J,K)*DRhoDPres(I,J,K)/Delta_t
            if (PresNodeTypes(I,J,K)=='F') then
               if ((NumNodesX>1).and.(K>1).and.(K<NumNodesZ)) then
                  Source = Source + PresBoundVals(I,J,K)/ &
                           XNodeWidths(I)
               else
                  Source = Source + PresBoundVals(I,J,K)/ &
                           ZNodeWidths(K)
               end if
            end if
            PresCoeffs(I,J,K,8)=Source
         elseif (PresNodeTypes(I,J,K)=='C') then
            PresCoeffs(I,J,K,1)=1.D0
            PresCoeffs(I,J,K,2)=0.D0
            PresCoeffs(I,J,K,3)=0.D0
            PresCoeffs(I,J,K,4)=0.D0
            PresCoeffs(I,J,K,5)=0.D0
            PresCoeffs(I,J,K,6)=0.D0
            PresCoeffs(I,J,K,7)=0.D0
            PresCoeffs(I,J,K,8)=PresBoundVals(I,J,K)
         end if
      end do
   end do
end do

end
