subroutine GetQP_Avg(I,J,K,QP_Avg)
! Purpose: To calculate the QP term in the pressure equation.
! Input: node indices I,J,K
! Output: QP_Avg, part of the source term in the pressure
!         equation.
! Note: DistPE is the distance between nodes P and E, as defined
! in Patankar (1980), and so on...

use CommonVars, only: Prec,Porosities,BulkDens,Delta_t,PresGamma2Z,  &
   ZNodeWidths,NumNodesZ,Sali,OldSali,Temp,OldTemp,DRhoDSali,        &
   DRhoDTemp,OldPoros,PresGamma2X,Tilt,NumNodesX,XNodeWidths,        &
   TimeStepFlag,LogFile,SourceAdjust,OldDelta_t,YNodeWidths,Pres,    &
   OldPres,DRhoDPres,DeltaMassGlobal,SystemVolume,SystemVolume,      &
   LocalSaltError,LocalMassError,LocalEnergyError,SaltTol,MassTol,   &
   EnergyTol
implicit none
integer :: I,J,K,Increment = 0
real (selected_real_kind(Prec)) :: QP_Avg, DRhoDt,Term1,Term2,       &
   DGamma2Dz,DSaliDt,DTempDt,DGamma2Dx,SaliTerm,TempTerm,DensTerm,   &
   DPhiDt,TimeStep,Correct,Scaling,ErrorRatio

if (SourceAdjust==0) then
   TimeStep = Delta_t
else
   TimeStep = OldDelta_t
end if

DSaliDt = (Sali(I,J,K)-OldSali(I,J,K))/TimeStep
DTempDt = (Temp(I,J,K)-OldTemp(I,J,K))/TimeStep
DPhiDt  = (Porosities(I,J,K)-OldPoros(I,J,K))/TimeStep
TempTerm = Porosities(I,J,K)*DRhoDTemp(I,J,K)
SaliTerm = Porosities(I,J,K)*DRhoDSali(I,J,K)
DensTerm = BulkDens(I,J,K)

Term1 = -(DTempDt*TempTerm + DSaliDt*SaliTerm + &
          DPhiDt*DensTerm)

if (K>1.and.K<NumNodesZ) then
   DGamma2Dz = (PresGamma2Z(I,J,K)-PresGamma2Z(I,J,K-1))/ &
               ZNodeWidths(K)
elseif (K==1) then
   DGamma2Dz = PresGamma2Z(I,J,K)/ZNodeWidths(K)
elseif (K==NumNodesZ) then
   DGamma2Dz = -PresGamma2Z(I,J,K-1)/ZNodeWidths(K)
end if

if ((Tilt>0.D0).and.(NumNodesX>1)) then
   if (I>1.and.I<NumNodesX) then
      DGamma2Dx = (PresGamma2X(I,J,K)-PresGamma2X(I-1,J,K))/ &
                   XNodeWidths(I)
   elseif (I==1) then
      DGamma2Dx = PresGamma2X(I,J,K)/XNodeWidths(I)
   elseif (I==NumNodesX) then
      DGamma2Dx = -PresGamma2X(I-1,J,K)/XNodeWidths(I)
   end if
else
   DGamma2Dx=0.D0
end if

Term2 = -DGamma2Dz-DGamma2Dx

ErrorRatio = MAX(LocalMassError/MassTol,LocalSaltError/SaltTol)
ErrorRatio = MAX(ErrorRatio,LocalEnergyError/EnergyTol)
ErrorRatio = MIN(ErrorRatio,1.D0)
if (ErrorRatio.ne.0.D0) then
   Scaling = 1.D0-exp(1.D0-1.D0/ErrorRatio)
else
   Scaling = 1.D0
end if
Correct = Scaling*DeltaMassGlobal/(Delta_t*SystemVolume)

QP_Avg = Term1+Term2+Correct
end
