subroutine GetQS_Avg(I,J,K,QS_Avg)
! Purpose: To calculate the QS term in the salinity equation.
! Input: Node indices I,J,K
! Output: QS_Avg, the source term for the salinity equation.

use CommonVars, only: Prec,XNodeWidths,YNodeWidths,ZNodeWidths,      &
   NumNodesX,NumNodesY,NumNodesZ,XInterfMassFlux,ZInterfMassFlux,    &
   XInterfSaltFlux,ZInterfSaltFlux,OldPoros,BulkDens,OldSali,        &
   Delta_t,SaliNodeTypes,Porosities,SaliBoundVals,                   &
   PhaseIndices,TimeStepCheck,TimeStepFlag,NumTimeCuts,MaxTimeCuts,  &
   LogFile,Time,SaltErrNum,StepNum,PhaseIndices,SourceAdjust,        &
   OldDelta_t,SaltTol,DeltaSaltGlobal,SystemVolume,DeltaSaltLocal,   &
   LocalSaltError,LocalMassError,LocalEnergyError,MassTol,SaltTol,   &
   EnergyTol
implicit none
real (selected_real_kind(Prec)) :: QS_Avg,TotalSalt,TotalMass,       &
   TotalVolume,TotalMassIn,TotalSaltIn,TimeStep,Salin,SalinCorrect,  &
   MaxPercent,PercentChange,SaltError,MassIn,SaltIn,Correct,Scaling, &
   ErrorRatio
integer :: I,J,K

TimeStep = Delta_t
!--------------------------------------------------------------------
TotalVolume = XNodeWidths(I)*YNodeWidths(J)*ZNodeWidths(K)

TotalMassIn = MassIn(I,J,K)
TotalSaltIn = SaltIn(I,J,K)

TotalSalt = BulkDens(I,J,K)*OldSali(I,J,K)*OldPoros(I,J,K)* &
            TotalVolume + TotalSaltIn
TotalMass = BulkDens(I,J,K)*OldPoros(I,J,K)*TotalVolume + &
            TotalMassIn

Salin = TotalSalt/TotalMass
if (OldSali(I,J,K)>0.D0) then
   PercentChange = MIN(100.D0*DABS(Salin-OldSali(I,J,K))/ &
      OldSali(I,J,K),100.D0)
else
   PercentChange = 0.D0
end if
MaxPercent = 1.D-4
if ((PercentChange>MaxPercent).or.(OldSali(I,J,K)==0.D0).or. &
   (SaltError(I,J,K,Salin)>SaltTol/100.D0)) then
   ErrorRatio = MAX(LocalMassError/MassTol,LocalSaltError/SaltTol)
   ErrorRatio = MAX(ErrorRatio,LocalEnergyError/EnergyTol)
   ErrorRatio = MIN(ErrorRatio,1.D0)
   if (ErrorRatio.ne.0.D0) then
      Scaling = 1.D0-exp(1.D0-1.D0/ErrorRatio)
   else
      Scaling = 1.D0
   end if
   Correct = Scaling*(DeltaSaltGlobal/SystemVolume)*TotalVolume
   TotalSaltIn = TotalSaltIn + Correct
   Salin = SalinCorrect(I,J,K,Salin,TotalSaltIn,TotalMassIn)
end if

QS_Avg = Salin
end
