subroutine GetQT_Avg(I,J,K,QT_Avg)
! Purpose: To calculate the QT term for the temperature equation.
! Input: Node indices I,J,K
! Output: QT_Avg, part of the source term in the temperature equation.

use CommonVars, only:Prec,BulkEnth,OldEnth,ZInterfMassFlux,          &
   XInterfMassFlux,NumNodesX,NumNodesY,NumNodesZ,BulkDens,           &
   Porosities,XNodeWidths,YNodeWidths,ZNodeWidths,Delta_t,           &
   TimeStepFlag,OldDelta_t,SourceAdjust,CpRock,RockDens,             &
   GlobalEnergyError,DeltaEnergyGlobal,SystemVolume,                 &
   LocalMassError,OldTemp,OldPres,OldSali,OldDens,EnergyTol,         &
   DeltaEnergyLocal,MassTol,SaltTol,EnergyTol,LocalSaltError,        &
   LocalEnergyError
implicit none
real (selected_real_kind(Prec)) :: QT_Avg,MassFluxTop,MassFluxBot,   &
     AvgEnth,AvgDens,MassFluxEast,MassFluxWest,DEnthDt,              &
     TimeStep,CalcBulkDens,EnthTerm1,EnthTerm2,DensTerm1,DensTerm2,  &
     Correct,EnergyError,Scaling,TotalVolume,Term1,Term2,ErrorRatio
integer :: I,J,K

if (SourceAdjust==0) then
   TimeStep = Delta_t
else
   TimeStep = OldDelta_t
end if

if (K==1) then
   MassFluxBot = ZInterfMassFlux(I,J,K)
   MassFluxTop = 0.D0
elseif (K==NumNodesZ) then
   MassFluxTop = ZInterfMassFlux(I,J,K-1)
   MassFluxBot = 0.D0
else
   MassFluxTop = ZInterfMassFlux(I,J,K-1)
   MassFluxBot = ZInterfMassFlux(I,J,K)
end if

if (I==1) then
   MassFluxEast = XInterfMassFlux(I,J,K)
   MassFluxWest = 0.D0
elseif (I==NumNodesX) then
   MassFluxWest = XInterfMassFlux(I-1,J,K)
   MassFluxEast = 0.D0
else
   MassFluxWest = XInterfMassFlux(I-1,J,K)
   MassFluxEast = XInterfMassFlux(I,J,K)
end if
DEnthDt = (BulkEnth(I,J,K)-OldEnth(I,J,K))/TimeStep
EnthTerm1 = BulkEnth(I,J,K)*(MassFluxBot-MassFluxTop)/ZNodeWidths(K)+ &
            BulkEnth(I,J,K)*(MassFluxEast-MassFluxWest)/XNodeWidths(I)
EnthTerm2 = OldEnth(I,J,K)*(MassFluxBot-MassFluxTop)/ZNodeWidths(K)+ &
            OldEnth(I,J,K)*(MassFluxEast-MassFluxWest)/XNodeWidths(I)
if (EnthTerm1>EnthTerm2) then
   if (DeltaEnergyGlobal>0.D0) then
      AvgEnth = BulkEnth(I,J,K)
   else
      AvgEnth = OldEnth(I,J,K)
   end if
else
   if (DeltaEnergyGlobal>0.D0) then
      AvgEnth = OldEnth(I,J,K)
   else
      AvgEnth = BulkEnth(I,J,K)
   end if
end if
DensTerm1 = -Porosities(I,J,K)*BulkDens(I,J,K)*DEnthDt
DensTerm2 = -Porosities(I,J,K)*OldDens(I,J,K)*DEnthDt
if (DensTerm1>DensTerm2) then
   if (DeltaEnergyGlobal>0.D0) then
      AvgDens = BulkDens(I,J,K)
   else
      AvgDens = OldDens(I,J,K)
   end if
else
   if (DeltaEnergyGlobal>0.D0) then
      AvgDens = OldDens(I,J,K)
   else
      AvgDens = BulkDens(I,J,K)
   end if
end if

Term1 = AvgEnth*(MassFluxBot-MassFluxTop)/ZNodeWidths(K)+ &
        AvgEnth*(MassFluxEast-MassFluxWest)/XNodeWidths(I)
Term2 = -Porosities(I,J,K)*AvgDens*DEnthDt

if (1.D2*EnergyError(I,J,K)>EnergyTol/1.D5) then
   ErrorRatio = MAX(LocalMassError/MassTol,LocalSaltError/SaltTol)
   ErrorRatio = MAX(ErrorRatio,LocalEnergyError/EnergyTol)
   ErrorRatio = MIN(ErrorRatio,1.D0)
   if (ErrorRatio.ne.0.D0) then
      Scaling = 1.D0-exp(1.D0-1.D0/ErrorRatio)
   else
      Scaling = 1.D0
   end if
   Correct = Scaling*DeltaEnergyGlobal/(Delta_t*SystemVolume)
   if (Correct>0.D0) then
      if (DeltaEnergyLocal(I,J,K)<0.D0) then
         Correct = 0.D0
      else
         TotalVolume = XNodeWidths(I)*YNodeWidths(J)*ZNodeWidths(K)
         Correct = MIN(Correct,DeltaEnergyLocal(I,J,K)/ &
            (Delta_t*TotalVolume))
      end if
   else
      if (DeltaEnergyLocal(I,J,K)>0.D0) then
         Correct = 0.D0
      else
         TotalVolume = XNodeWidths(I)*YNodeWidths(J)*ZNodeWidths(K)
         Correct = MAX(Correct,DeltaEnergyLocal(I,J,K)/ &
            (Delta_t*TotalVolume))
      end if
   end if
else
   Correct = 0.D0
end if

QT_Avg = Term1+Term2+Correct
end
