subroutine GetSaliCoeffs
! Purpose: To calculate the coefficients for the salinity equation.

use CommonVars, only:SaliCoeffs,NumNodesX,NumNodesY,NumNodesZ, &
    SaliNodeTypes,XNodeWidths,YNodeWidths,ZNodeWidths,         &
    Prec,Delta_t,OldSali,SaliBoundVals,ZInterfLiqDens,         &
    ZInterfVapDens,LiqVelocZ,VapVelocZ,ZInterfSaltFlux
implicit none
real (selected_real_kind(Prec)) :: aN,aS,aW,aE,aB,aT,QS_Avg,   &
     Source,Flux
integer :: I,J,K

do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         if ((SaliNodeTypes(I,J,K)=='I').or. &
            (SaliNodeTypes(I,J,K)=='F')) then
            !--------------------------------------------------------
            !                     coeff: aN
            !--------------------------------------------------------
            aN=0.D0
            SaliCoeffs(I,J,K,2)=aN
            !--------------------------------------------------------
            !                     coeff: aS
            !--------------------------------------------------------
            aS=0.D0
            SaliCoeffs(I,J,K,3)=aS
            !--------------------------------------------------------
            !                     coeff: aW
            !--------------------------------------------------------
            aW=0.D0
            SaliCoeffs(I,J,K,4)=aW
            !--------------------------------------------------------
            !                     coeff: aE
            !--------------------------------------------------------
            aE=0.D0
            SaliCoeffs(I,J,K,5)=aE
            !--------------------------------------------------------
            !                     coeff: aB
            !--------------------------------------------------------
            aB=0.D0
            SaliCoeffs(I,J,K,6)=aB
            !--------------------------------------------------------
            !                     coeff: aT
            !--------------------------------------------------------
            aT=0.D0
            SaliCoeffs(I,J,K,7)=aT
            !--------------------------------------------------------
            !                     coeff: aP
            !--------------------------------------------------------
            SaliCoeffs(I,J,K,1)=1.D0
            !--------------------------------------------------------
            !                     source term
            !--------------------------------------------------------
            call GetQS_Avg(I,J,K,QS_Avg)
            SaliCoeffs(I,J,K,8)=QS_Avg
         elseif (SaliNodeTypes(I,J,K)=='C') then
            SaliCoeffs(I,J,K,1)=1.D0
            SaliCoeffs(I,J,K,2)=0.D0
            SaliCoeffs(I,J,K,3)=0.D0
            SaliCoeffs(I,J,K,4)=0.D0
            SaliCoeffs(I,J,K,5)=0.D0
            SaliCoeffs(I,J,K,6)=0.D0
            SaliCoeffs(I,J,K,7)=0.D0
            SaliCoeffs(I,J,K,8)=SaliBoundVals(I,J,K)
         elseif (SaliNodeTypes(I,J,K)=='U') then
            SaliCoeffs(I,J,K,1)=1.D0
            SaliCoeffs(I,J,K,2)=0.D0
            SaliCoeffs(I,J,K,3)=0.D0
            SaliCoeffs(I,J,K,4)=0.D0
            SaliCoeffs(I,J,K,5)=0.D0
            SaliCoeffs(I,J,K,6)=0.D0
            SaliCoeffs(I,J,K,7)=0.D0
            Flux = ZInterfSaltFlux(I,J,K)
            if (Flux<0.D0) then
               SaliCoeffs(I,J,K,8)=OldSali(I,J,K+1)
            elseif (Flux>0.D0) then
               SaliCoeffs(I,J,K,8)=SaliBoundVals(I,J,K)
            else
               SaliCoeffs(I,J,K,8)=OldSali(I,J,K)
            end if
         end if
      end do
   end do
end do

end
