subroutine GetTempCoeffs
! Purpose: To calculate the coefficients for the temperature 
!          equation.

use CommonVars, only:TempCoeffs,NumNodesX,NumNodesY,NumNodesZ, &
   TempNodeTypes,XNodeWidths,YNodeWidths,ZNodeWidths,          &
   RockThermCond,Delta_t,Porosities,CpRock,RockDens,Prec,      &
   OldTemp,Temp,TempBoundVals,LiqVelocZ,VapVelocZ,             &
   ZInterfLiqEnth,ZInterfVapEnth,ZInterfLiqDens,               &
   ZInterfVapDens,VapVolSats,LiqVolSats,LiquidDens,VaporDens,  &
   LiquidEnth,VaporEnth,BulkDens,BulkEnth,OldEnth,             &
   FluidThermCond,XInterfLiqEnth,XInterfVapEnth,LiqVelocX,     &
   VapVelocX,XInterfLiqDens,XInterfVapDens,XInterfHeatFlux,    &
   ZInterfHeatFlux,XInterfTemp,ZInterfTemp,XInterfPoros,       &
   ZInterfPoros,OldPoros,ZInterfMassFlux,XInterfLiqCp,         &
   ZInterfLiqCp,XInterfVapCp,ZInterfVapCp
implicit none
real (selected_real_kind(Prec)) :: NodeDistance, aN,aS,aW,aE,  &
     aB,aT,QT_Avg,Source,Flux,Fzb,Fzt,Dzb,Dzt,aP,KMedium,Fxe,  &
     Fxw,Dxe,Dxw,DiffFlux,CrossArea
integer :: I,J,K

do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         if ((TempNodeTypes(I,J,K)=='I').or. &
            (TempNodeTypes(I,J,K)=='F')) then
            !--------------------------------------------------------
            !                     coeff: aN
            !--------------------------------------------------------
            if (J>1) then ! there is a node to the north
               aN=0.D0
            else
               aN=0.D0
            end if
            TempCoeffs(I,J,K,2)=aN
            !--------------------------------------------------------
            !                     coeff: aS
            !--------------------------------------------------------
            if (J<NumNodesY) then ! there is a node to the south
               aS=0.D0
            else
               aS=0.D0
            end if
            TempCoeffs(I,J,K,3)=aS
            !--------------------------------------------------------
            !                     coeff: aW
            !--------------------------------------------------------
            if (I>1) then ! there is a node to the west
               NodeDistance = XNodeWidths(I-1)/2.D0             &
                              + XNodeWidths(I)/2.D0
               KMedium=XInterfPoros(I-1,J,K)*FluidThermCond + &
                       (1.D0-XInterfPoros(I-1,J,K))*RockThermCond
               Fxw = XInterfLiqDens(I-1,J,K)*LiqVelocX(I-1,J,K)* &
                     XInterfLiqCp(I-1,J,K) + &
                     XInterfVapDens(I-1,J,K)*VapVelocX(I-1,J,K)* &
                     XInterfVapCp(I-1,J,K)
               Dxw = KMedium/(NodeDistance*XNodeWidths(I))
               aW=Dxw + MAX(Fxw,0.D0)/XNodeWidths(I)
            else
               Fxw=0.D0
               Dxw=0.D0
               aW=0.D0
            end if
            TempCoeffs(I,J,K,4)=aW
            !--------------------------------------------------------
            !                     coeff: aE
            !--------------------------------------------------------
            if (I<NumNodesX) then ! there is a node to the east
               NodeDistance = XNodeWidths(I+1)/2.D0             &
                              + XNodeWidths(I)/2.D0
               KMedium=XInterfPoros(I,J,K)*FluidThermCond + &
                       (1.D0-XInterfPoros(I,J,K))*RockThermCond
               Fxe = XInterfLiqDens(I,J,K)*LiqVelocX(I,J,K)* &
                     XInterfLiqCp(I,J,K) + &
                     XInterfVapDens(I,J,K)*VapVelocX(I,J,K)* &
                     XInterfVapCp(I,J,K)
               Dxe = KMedium/(NodeDistance*XNodeWidths(I))
               aE=Dxe + MAX(-Fxe,0.D0)/XNodeWidths(I)
            else
               Fxe=0.D0
               Dxe=0.D0
               aE=0.D0
            end if
            TempCoeffs(I,J,K,5)=aE
            !--------------------------------------------------------
            !                     coeff: aB
            !--------------------------------------------------------
            if (K<NumNodesZ) then ! there is a node below
               NodeDistance = ZNodeWidths(K+1)/2.D0             &
                              + ZNodeWidths(K)/2.D0
               KMedium=ZInterfPoros(I,J,K)*FluidThermCond + &
                       (1.D0-ZInterfPoros(I,J,K))*RockThermCond
               Fzb = ZInterfLiqDens(I,J,K)*LiqVelocZ(I,J,K)* &
                     ZInterfLiqCp(I,J,K) + &
                     ZInterfVapDens(I,J,K)*VapVelocZ(I,J,K)* &
                     ZInterfVapCp(I,J,K)
               Dzb = KMedium/(NodeDistance*ZNodeWidths(K))
               aB=Dzb + MAX(-Fzb,0.D0)/ZNodeWidths(K)
            else
               Fzb=0.D0
               Dzb=0.D0
               aB=0.D0
            end if
            TempCoeffs(I,J,K,6)=aB
            !--------------------------------------------------------
            !                     coeff: aT
            !--------------------------------------------------------
            if (K>1) then ! there is a node above
               NodeDistance = ZNodeWidths(K-1)/2.D0             &
                              + ZNodeWidths(K)/2.D0
               KMedium=ZInterfPoros(I,J,K-1)*FluidThermCond + &
                       (1.D0-ZInterfPoros(I,J,K-1))*RockThermCond
               Fzt = ZInterfLiqDens(I,J,K-1)*LiqVelocZ(I,J,K-1)* &
                     ZInterfLiqCp(I,J,K-1) + &
                     ZInterfVapDens(I,J,K-1)*VapVelocZ(I,J,K-1)* &
                     ZInterfVapCp(I,J,K-1)
               Dzt = KMedium/(NodeDistance*ZNodeWidths(K))
               aT=Dzt + MAX(Fzt,0.D0)/ZNodeWidths(K)
            else
               Fzt=0.D0
               Dzt=0.D0
               aT=0.D0
            end if
            TempCoeffs(I,J,K,7)=aT
            !--------------------------------------------------------
            !                     coeff: aP
            !--------------------------------------------------------
            aP = Dzb+Dzt+(MAX(Fzb,0.D0)+MAX(-Fzt,0.D0))/ &
               ZNodeWidths(K)+ &
               Dxe+Dxw+(MAX(Fxe,0.D0)+MAX(-Fxw,0.D0))/ &
               XNodeWidths(I)+ &
               (1.D0-Porosities(I,J,K))*RockDens*CpRock/Delta_t
            TempCoeffs(I,J,K,1)=aP
            !--------------------------------------------------------
            !                     source term
            !--------------------------------------------------------
            call GetQT_Avg(I,J,K,QT_Avg)
            Source = QT_Avg+(1.D0-OldPoros(I,J,K))*RockDens*       &
                     CpRock*OldTemp(I,J,K)/Delta_t
            if (TempNodeTypes(I,J,K)=='F') then
               if ((K==1).or.(K==NumNodesZ)) then
                  TempCoeffs(I,J,K,8)=Source + TempBoundVals(I,J,K)/ &
                                      ZNodeWidths(K)
               elseif ((J==1).or.(J==NumNodesY)) then
                  TempCoeffs(I,J,K,8)=Source + TempBoundVals(I,J,K)/ &
                                      YNodeWidths(J)
               elseif ((I==1).or.(I==NumNodesX)) then
                  TempCoeffs(I,J,K,8)=Source + TempBoundVals(I,J,K)/ &
                                      XNodeWidths(I)
               end if
            else
               TempCoeffs(I,J,K,8)=Source
            end if
         elseif (TempNodeTypes(I,J,K)=='C') then
            TempCoeffs(I,J,K,1)=1.D0
            TempCoeffs(I,J,K,2)=0.D0
            TempCoeffs(I,J,K,3)=0.D0
            TempCoeffs(I,J,K,4)=0.D0
            TempCoeffs(I,J,K,5)=0.D0
            TempCoeffs(I,J,K,6)=0.D0
            TempCoeffs(I,J,K,7)=0.D0
            TempCoeffs(I,J,K,8)=TempBoundVals(I,J,K)
         elseif (TempNodeTypes(I,J,K)=='U') then
            TempCoeffs(I,J,K,1)=1.D0
            TempCoeffs(I,J,K,2)=0.D0
            TempCoeffs(I,J,K,3)=0.D0
            TempCoeffs(I,J,K,4)=0.D0
            TempCoeffs(I,J,K,5)=0.D0
            TempCoeffs(I,J,K,6)=0.D0
            TempCoeffs(I,J,K,7)=0.D0
            if (K < NumNodesZ) then
               NodeDistance = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
               KMedium=ZInterfPoros(I,1,K)*FluidThermCond + &
                  (1.D0-ZInterfPoros(I,1,K))*RockThermCond
               DiffFlux = (KMedium*(Temp(I,J,K+1)-Temp(I,J,K))/ &
                  NodeDistance)
            else
               DiffFlux = 0.D0
            end if
            Flux = DiffFlux - ZInterfHeatFlux(I,J,K)
            if (Flux>0.D0) then
               TempCoeffs(I,J,K,8)=OldTemp(I,J,K+1)
            elseif (Flux<0.D0) then
               TempCoeffs(I,J,K,8)=TempBoundVals(I,J,K)
            else
               TempCoeffs(I,J,K,8)=OldTemp(I,J,K)
            end if
         end if
      end do
   end do
end do
end
