subroutine GetVelocities
! Purpose: To calculate the fluid velocity (m/s) at each node 
!          interface.
! Note: Positive Z velocity indicates fluid motion in the positive
! Z direction - that is, downward; positive X velocity indicates
! fluid motion in the positive X direction - that is, from  west
! to east.

use CommonVars, only:Prec,VapVelocX,VapVelocY,VapVelocZ,LiqVelocX,     &
    LiqVelocY,LiqVelocZ,NumNodesX,NumNodesY,NumNodesZ,InterfPermsX,    &
    InterfPermsZ,XInterfLiqVolSats,ZInterfLiqVolSats,VaporSali,        &
    LiquidSali,Pres,Temp,XNodeWidths,YNodeWidths,ZNodeWidths,          &
    Grav,XInterfVapDens,XInterfLiqDens,ZInterfVapDens,ZInterfLiqDens,  &
    PresGamma1X,PresGamma1Y,PresGamma1Z,PresGamma2Z,XInterfPres,       &
    ZInterfPres,BulkDens,ZInterfTemp,ZInterfLiqSali,ZInterfVapSali,    &
    PhaseIndices,XInterfTemp,XInterfLiqSali,XInterfVapSali,Tilt,       &
    PresGamma2X,XInterfVapVolSats,ZInterfVapVolSats,Delta_t,           &
    Porosities,LogFile,TimeStepFlag,TimeStepCheck,NumTimeCuts,         &
    MaxTimeCuts
    
implicit none
real (selected_real_kind(Prec)) :: RelPermL,RelPermV,LiqDynVisc,       &
    VapDynVisc,PresGrad,FViscosity,NodeDistance,GammaTerm1,GammaTerm2, &
    Perm,NodeDist,DeltaPres,InterfLiqDens,InterfVapDens,RelInterfPos,  &
    LiqDen1,LiqDen2,AvgLiqDen,VapDen1,VapDen2,AvgVapDen
integer I,J,K
!----------------------------
! X direction velocities
!----------------------------
if ((NumNodesX-1).ne.0) then
   do I=1,NumNodesX-1
      do J=1,NumNodesY
         do K=1,NumNodesZ
            Perm = InterfPermsX(I,J,K)
            RelPermL = XInterfLiqVolSats(I,J,K)
            RelPermV = XInterfVapVolSats(I,J,K)
            LiqDynVisc = FViscosity(XInterfTemp(I,J,K),      &
                         XInterfPres(I,J,K),XInterfLiqSali(I,J,K))
            VapDynVisc = FViscosity(XInterfTemp(I,J,K),      &
                         XInterfPres(I,J,K),XInterfVapSali(I,J,K))

            DeltaPres = Pres(I+1,J,K)-Pres(I,J,K)
            NodeDist = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
            PresGrad = DeltaPres/NodeDist
            LiqVelocX(I,J,K) = -(Perm*RelPermL/LiqDynVisc)* &
                             (PresGrad-XInterfLiqDens(I,J,K)*Grav &
                             *sin(Tilt))
            VapVelocX(I,J,K) = -(Perm*RelPermV/VapDynVisc)* &
                             (PresGrad-XInterfVapDens(I,J,K)*Grav &
                             *sin(Tilt))
            if ((TimeStepFlag==0).and.(TimeStepCheck=='Y')) then
               if ((LiqVelocX(I,J,K)*Delta_t>=XNodeWidths(I)).or. &
                   (VapVelocX(I,J,K)*Delta_t>=XNodeWidths(I))) then
                  write(LogFile,*) 'Fluid crosses some control volume'
                  write(LogFile,*) 'in one time step'
                  TimeStepFlag = 1
               end if
            end if
            ! Calculate interface values of Gammas (pres eq.)
            GammaTerm1=(Perm*RelPermV*XInterfVapDens(I,J,K)) &
                       /VapDynVisc
            GammaTerm2=(Perm*RelPermL*XInterfLiqDens(I,J,K)) &
                       /LiqDynVisc
            PresGamma1X(I,J,K) = (GammaTerm1+GammaTerm2)
            !--------------------------------------------------------
            !                     Gamma 2, x direction
            !--------------------------------------------------------
            if ((Tilt>0.D0).and.(NumNodesX>1)) then
               GammaTerm1=GammaTerm1*XInterfVapDens(I,J,K)*Grav*sin(Tilt)
               GammaTerm2=GammaTerm2*XInterfLiqDens(I,J,K)*Grav*sin(Tilt)
               PresGamma2X(I,J,K) = (GammaTerm1+GammaTerm2)
            else
               PresGamma2X(I,J,K) = 0.D0
            end if
         end do
      end do
   end do
end if
!----------------------------
! Y direction velocities
!----------------------------
! Add if ever want the 3rd dimension
!----------------------------
! Z direction velocities
!----------------------------
if ((NumNodesZ-1).ne.0) then
   do K=1,NumNodesZ-1
      do I=1,NumNodesX
         do J=1,NumNodesY
            Perm = InterfPermsZ(I,J,K)
            RelPermL = ZInterfLiqVolSats(I,J,K)
            RelPermV = ZInterfVapVolSats(I,J,K)
            LiqDynVisc = FViscosity(ZInterfTemp(I,J,K),      &
                         ZInterfPres(I,J,K),ZInterfLiqSali(I,J,K))
            VapDynVisc = FViscosity(ZInterfTemp(I,J,K),      &
                         ZInterfPres(I,J,K),ZInterfVapSali(I,J,K))
 
            DeltaPres = Pres(I,J,K+1)-Pres(I,J,K)
            NodeDist = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
            PresGrad = DeltaPres/NodeDist
            LiqVelocZ(I,J,K) = -(Perm*RelPermL/LiqDynVisc)*  &
                             (PresGrad-ZInterfLiqDens(I,J,K)*Grav &
                             *cos(Tilt))
            VapVelocZ(I,J,K) = -(Perm*RelPermV/VapDynVisc)*  &
                             (PresGrad-ZInterfVapDens(I,J,K)*Grav &
                             *cos(Tilt))
            if ((TimeStepFlag==0).and.(TimeStepCheck=='Y')) then
               if ((LiqVelocZ(I,J,K)*Delta_t>=ZNodeWidths(K)).or. &
                   (VapVelocZ(I,J,K)*Delta_t>=ZNodeWidths(K))) then
                  write(LogFile,*) 'Fluid crosses some control volume'
                  write(LogFile,*) 'in one time step'
                  TimeStepFlag = 1
               end if
            end if
            ! Calculate interface values of Gammas (pres eq.)
            GammaTerm1=(Perm*RelPermV*ZInterfVapDens(I,J,K)) &
                       /VapDynVisc
            GammaTerm2=(Perm*RelPermL*ZInterfLiqDens(I,J,K)) &
                       /LiqDynVisc
            PresGamma1Z(I,J,K) = (GammaTerm1+GammaTerm2)
            !--------------------------------------------------------
            !                     Gamma 2, z direction
            !--------------------------------------------------------
            GammaTerm1=GammaTerm1*ZInterfVapDens(I,J,K)*Grav*cos(Tilt)
            GammaTerm2=GammaTerm2*ZInterfLiqDens(I,J,K)*Grav*cos(Tilt)
            PresGamma2Z(I,J,K) = (GammaTerm1+GammaTerm2)
         end do
      end do
   end do
end if
end
