subroutine InterfSaltFlux

use CommonVars, only: Prec,NumNodesX,NumNodesY,NumNodesZ,VapVelocX, &
   LiqVelocX,VapVelocZ,LiqVelocZ,XInterfSaltFlux,ZInterfSaltFlux,   &
   XInterfVapSali,ZInterfVapSali,XInterfLiqSali,ZInterfLiqSali,     &
   XInterfVapDens,XInterfLiqDens,ZInterfVapDens,ZInterfLiqDens,     &
   SaltChemDiff,CorrelationLength,Dispersivity,XInterfLiqSaltGrad,  &
   XInterfVapSaltGrad,ZInterfLiqSaltGrad,ZInterfVapSaltGrad,        &
   XSaltDiffFlux,ZSaltDiffFlux
implicit none
real(selected_real_kind(Prec)) :: LGamma,VGamma
integer :: I,J,K

! note: diffusion is included in the interface salt fluxes, but not
! in the interface heat or mass fluxes, because the heat equation
! coefficients already contain diffusion implicitly in the case of
! the former, and because including mass diffusion makes no sense
! if mass = total fluid mass (at least not if what diffuses is 
! "proportion of marked molecules", as in GK Batchelor's text)
!--------------------------------------------------------------------
! salt fluxes in the x direction
!--------------------------------------------------------------------
call InterfSaltGrads
do I=1,NumNodesX-1
   do J=1,NumNodesY
      do K=1,NumNodesZ
         LGamma = SaltChemDiff + Dispersivity*CorrelationLength* &
                  LiqVelocX(I,J,K)
         VGamma = SaltChemDiff + Dispersivity*CorrelationLength* &
                  VapVelocX(I,J,K)
         XSaltDiffFlux(I,J,K) = -LGamma*XInterfLiqSaltGrad(I,J,K) &
                                -VGamma*XInterfVapSaltGrad(I,J,K)

         XInterfSaltFlux(I,J,K) = &
            VapVelocX(I,J,K)*XInterfVapDens(I,J,K)* &
            XInterfVapSali(I,J,K) + &
            LiqVelocX(I,J,K)*XInterfLiqDens(I,J,K)* &
            XInterfLiqSali(I,J,K) + XSaltDiffFlux(I,J,K)
      end do
   end do
end do
!--------------------------------------------------------------------
! salt fluxes in the z direction
!--------------------------------------------------------------------
do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ-1
         LGamma = SaltChemDiff + Dispersivity*CorrelationLength* &
                  LiqVelocZ(I,J,K)
         VGamma = SaltChemDiff + Dispersivity*CorrelationLength* &
                  VapVelocZ(I,J,K)
         ZSaltDiffFlux(I,J,K) = -LGamma*ZInterfLiqSaltGrad(I,J,K) &
                                -VGamma*ZInterfVapSaltGrad(I,J,K)

         ZInterfSaltFlux(I,J,K) = &
            VapVelocZ(I,J,K)*ZInterfVapDens(I,J,K)* &
            ZInterfVapSali(I,J,K) + &
            LiqVelocZ(I,J,K)*ZInterfLiqDens(I,J,K)* &
            ZInterfLiqSali(I,J,K) + ZSaltDiffFlux(I,J,K)
      end do
   end do
end do

end
