subroutine InterfSaltGrads
! Purpose: to calculate the gradients necessary to determine the
! diffusive salt fluxes at node interfaces
! note: the resulting quantities are not salinity gradients; they
! are gradients in salt concentration (kg/m^3)

! note also: DistPE is the distance between nodes P and E, as 
! defined in Patankar (1980), and so on...

use CommonVars, only: Prec,XInterfLiqSaltGrad,ZInterfLiqSaltGrad, &
   XInterfVapSaltGrad,ZInterfVapSaltGrad,NumNodesX,NumNodesY,     &
   NumNodesZ,XNodeWidths,YNodeWidths,ZNodeWidths,LiquidSali,      &
   VaporSali,XInterfPoros,ZInterfPoros,XInterfLiqDens,            &
   ZInterfLiqDens,XInterfVapDens,ZInterfVapDens
implicit none
real(selected_real_kind(Prec)) :: DistPE,DistPB
integer :: I,J,K

do K=1,NumNodesZ
   do J=1,NumNodesY
      do I=1,NumNodesX-1
         DistPE = (XNodeWidths(I)+XNodeWidths(I+1))/2.D0
         XInterfLiqSaltGrad(I,J,K) = &
            XInterfPoros(I,J,K)*XInterfLiqDens(I,J,K)* &
            (LiquidSali(I+1,J,K)-LiquidSali(I,J,K))/DistPE
         XInterfVapSaltGrad(I,J,K) = &
            XInterfPoros(I,J,K)*XInterfVapDens(I,J,K)* &
            (VaporSali(I+1,J,K)-VaporSali(I,J,K))/DistPE
      end do
   end do
end do

do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ-1
         DistPB = (ZNodeWidths(K)+ZNodeWidths(K+1))/2.D0
         ZInterfLiqSaltGrad(I,J,K) = &
            ZInterfPoros(I,J,K)*ZInterfLiqDens(I,J,K)* &
            (LiquidSali(I,J,K+1)-LiquidSali(I,J,K))/DistPB
         ZInterfVapSaltGrad(I,J,K) = &
            ZInterfPoros(I,J,K)*ZInterfVapDens(I,J,K)* &
            (VaporSali(I,J,K+1)-VaporSali(I,J,K))/DistPB
      end do
   end do
end do

end
