recursive function Interp3D(Tempr,Presr,Salin,InterpMat) &
                   Result(Answer)
! Input: temperature (C), pressure (Pa), salinity (wt frac
!        NaCl, and the 3D lookup matrix (DensSlices or
!        EnthSlices).
! Output: interpolated value in the lookup matrix (either
!         the bulk density or the bulk enthalpy).

use CommonVars, only: TempArray,PresArray,SaliArray,Prec
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Salin,Answer,T1,T2, &
   AnsT1,AnsT2,TempFrac,P1,P2,AnsP1,AnsP2,PresFrac,S1,S2,         &
   SaliFrac,AnsS1,AnsS2,InterpMat(79,200,100)
integer :: TempRow1,TempRow2,PresRow1,PresRow2, &
   SaliRow1,SaliRow2

call FindTemp(Tempr,TempRow1,TempRow2)
T1 = TempArray(TempRow1)
T2 = TempArray(TempRow2)
if (Tempr==T1) then
   call FindPres1Ph(Presr,PresRow1,PresRow2)
   P1 = PresArray(PresRow1)*1.D5
   P2 = PresArray(PresRow2)*1.D5
   if (Presr==P1) then
      call FindSali1Ph(Salin,SaliRow1,SaliRow2)
      S1 = SaliArray(SaliRow1)/1.D2
      S2 = SaliArray(SaliRow2)/1.D2
      if (Salin==S1) then
         Answer = InterpMat(TempRow1,PresRow1,SaliRow1)
      elseif (Salin==S2) then
         Answer = InterpMat(TempRow1,PresRow1,SaliRow2)
      else
         AnsS1 = Interp3D(T1,P1,S1,InterpMat)
         AnsS2 = Interp3D(T1,P1,S2,InterpMat)
         SaliFrac = (Salin-S1)/(S2-S1)
         Answer = (1.D0-SaliFrac)*AnsS1 + SaliFrac*AnsS2
      end if
   elseif (Presr==P2) then
      call FindSali1Ph(Salin,SaliRow1,SaliRow2)
      S1 = SaliArray(SaliRow1)/1.D2
      S2 = SaliArray(SaliRow2)/1.D2
      if (Salin==S1) then
         Answer = InterpMat(TempRow1,PresRow2,SaliRow1)
      elseif (Salin==S2) then
         Answer = InterpMat(TempRow1,PresRow2,SaliRow2)
      else
         AnsS1 = Interp3D(T1,P2,S1,InterpMat)
         AnsS2 = Interp3D(T1,P2,S2,InterpMat)
         SaliFrac = (Salin-S1)/(S2-S1)
         Answer = (1.D0-SaliFrac)*AnsS1 + SaliFrac*AnsS2
      end if
   else
      AnsP1 = Interp3D(T1,P1,Salin,InterpMat)
      AnsP2 = Interp3D(T1,P2,Salin,InterpMat)
      PresFrac = (Presr-P1)/(P2-P1)
      Answer = (1.D0-PresFrac)*AnsP1 + PresFrac*AnsP2
   end if
elseif (Tempr==T2) then
   call FindPres1Ph(Presr,PresRow1,PresRow2)
   P1 = PresArray(PresRow1)*1.D5
   P2 = PresArray(PresRow2)*1.D5
   if (Presr==P1) then
      call FindSali1Ph(Salin,SaliRow1,SaliRow2)
      S1 = SaliArray(SaliRow1)/1.D2
      S2 = SaliArray(SaliRow2)/1.D2
      if (Salin==S1) then
         Answer = InterpMat(TempRow2,PresRow1,SaliRow1)
      elseif (Salin==S2) then
         Answer = InterpMat(TempRow2,PresRow1,SaliRow2)
      else
         AnsS1 = Interp3D(T2,P1,S1,InterpMat)
         AnsS2 = Interp3D(T2,P1,S2,InterpMat)
         SaliFrac = (Salin-S1)/(S2-S1)
         Answer = (1.D0-SaliFrac)*AnsS1 + SaliFrac*AnsS2
      end if
   elseif (Presr==P2) then
      call FindSali1Ph(Salin,SaliRow1,SaliRow2)
      S1 = SaliArray(SaliRow1)/1.D2
      S2 = SaliArray(SaliRow2)/1.D2
      if (Salin==S1) then
         Answer = InterpMat(TempRow2,PresRow2,SaliRow1)
      elseif (Salin==S2) then
         Answer = InterpMat(TempRow2,PresRow2,SaliRow2)
      else
         AnsS1 = Interp3D(T2,P2,S1,InterpMat)
         AnsS2 = Interp3D(T2,P2,S2,InterpMat)
         SaliFrac = (Salin-S1)/(S2-S1)
         Answer = (1.D0-SaliFrac)*AnsS1 + SaliFrac*AnsS2
      end if
   else
      AnsP1 = Interp3D(T2,P1,Salin,InterpMat)
      AnsP2 = Interp3D(T2,P2,Salin,InterpMat)
      PresFrac = (Presr-P1)/(P2-P1)
      Answer = (1.D0-PresFrac)*AnsP1 + PresFrac*AnsP2
   end if
else
   AnsT1 = Interp3D(T1,Presr,Salin,InterpMat)
   AnsT2 = Interp3D(T2,Presr,Salin,InterpMat)
   TempFrac = (Tempr-T1)/(T2-T1)
   Answer = (1.D0-TempFrac)*AnsT1+TempFrac*AnsT2
end if

end
