subroutine MassTrackGlobal(InitVars)

use CommonVars, only: NumNodesZ,ZInterfMassFlux,MassInTop,           &
   InitSystemMass,FinlSystemMass,ZNodeWidths,XNodeWidths,            &
   YNodeWidths,BulkDens,Sali,Delta_t,Prec,Porosities,Time,MassInBot, &
   MassInWest,MassInEast,NumNodesX,XInterfMassFlux,TimeStepCheck,    &
   TimeStepFlag,NumTimeCuts,MaxTimeCuts,LogFile,GlobalMassTol,       &
   StepNum,Pres,Temp,LastPrintTime,PrintFreq,DeltaMassGlobal,        &
   GlobalMassError,PresNodeTypes,PresBoundVals
implicit none
real(selected_real_kind(Prec)) :: TotalMassIn,MassGain,tInYears,     &
   MassError,Volume,CalcBulkDens
integer :: PrintStep,InitVars,I,K

if (InitVars==1) then
   GlobalMassTol = 5.D0
   open(unit=14,file='MassTrackGlobal.dat',status='replace')
   close(14)
   MassInTop  = 0.D0
   MassInBot  = 0.D0
   MassInWest = 0.D0
   MassInEast = 0.D0
   InitSystemMass = 0.D0
   do K=1,NumNodesZ
      do I=1,NumNodesX
         if ((PresNodeTypes(I,1,K).ne.'U').and. &
            (PresNodeTypes(I,1,K).ne.'C')) then
            Volume = XNodeWidths(I)*YNodeWidths(1)*ZNodeWidths(K)
            InitSystemMass = InitSystemMass + &
               BulkDens(I,1,K)*Volume*Porosities(I,1,K)
         end if
      end do
   end do
else
   FinlSystemMass = 0.D0
   do K=1,NumNodesZ
      do I=1,NumNodesX
         if ((PresNodeTypes(I,1,K).ne.'U').and. &
            (PresNodeTypes(I,1,K).ne.'C')) then
            Volume = XNodeWidths(I)*YNodeWidths(1)*ZNodeWidths(K)
            FinlSystemMass = FinlSystemMass + &
               CalcBulkDens(Temp(I,1,K),Pres(I,1,K),Sali(I,1,K))* &
               Volume*Porosities(I,1,K)
         end if
         if (I>1) then
            if ((PresNodeTypes(I-1,1,K)=='U').or. &
               (PresNodeTypes(I-1,1,K)=='C')) then
               MassInWest = MassInWest + &
                  XInterfMassFlux(I-1,1,K)*Delta_t* &
                  ZNodeWidths(K)*YNodeWidths(1)
            elseif (PresNodeTypes(I-1,1,K)=='F') then
               if ((K.ne.1).and.(K.ne.NumNodesZ)) then
                  MassInWest = MassInWest + PresBoundVals(I-1,1,K)* &
                     ZNodeWidths(K)*Delta_t
               end if
            end if
         end if
         if (I<NumNodesX) then
            if ((PresNodeTypes(I+1,1,K)=='U').or. &
               (PresNodeTypes(I+1,1,K)=='C')) then
               MassInEast = MassInEast - &
                  XInterfMassFlux(I,1,K)*Delta_t* &
                  ZNodeWidths(K)*YNodeWidths(1)
            elseif (PresNodeTypes(I+1,1,K)=='F') then
               if ((K.ne.1).and.(K.ne.NumNodesZ)) then
                  MassInEast = MassInEast + PresBoundVals(I+1,1,K)* &
                     ZNodeWidths(K)*Delta_t
               end if
            end if
         end if
         if (K>1) then
            if ((PresNodeTypes(I,1,K-1)=='U').or. &
               (PresNodeTypes(I,1,K-1)=='C')) then
               MassInTop = MassInTop + &
                  ZInterfMassFlux(I,1,K-1)*Delta_t* &
                  XNodeWidths(I)*YNodeWidths(1)
            elseif (PresNodeTypes(I,1,K-1)=='F') then
               MassInTop = MassInTop + PresBoundVals(I,1,K-1)* &
                  XNodeWidths(I)*Delta_t
            end if
         end if
         if (K<NumNodesZ) then
            if ((PresNodeTypes(I,1,K+1)=='U').or. &
               (PresNodeTypes(I,1,K+1)=='C')) then
               MassInBot = MassInBot - &
                  ZInterfMassFlux(I,1,K)*Delta_t*   &
                  XNodeWidths(I)*YNodeWidths(1)
            elseif (PresNodeTypes(I,1,K+1)=='F') then
               MassInBot = MassInBot + PresBoundVals(I,1,K+1)* &
                  XNodeWidths(I)*Delta_t
            end if
         end if
      end do
   end do
   TotalMassIn = MassInTop + MassInBot + MassInWest + MassInEast
   MassGain = FinlSystemMass - InitSystemMass
   MassError = DABS(MassGain-TotalMassIn)
   DeltaMassGlobal = TotalMassIn-MassGain
   MassError = 100.D0*MassError/InitSystemMass
   GlobalMassError = MassError/100.D0

   !write(*,*) 'Global mass error: ',MassError

   if ((TimeStepCheck == 'Y').and.(TimeStepFlag == 0)) then
      if (MassError > GlobalMassTol) then
         if ((NumTimeCuts < MaxTimeCuts).and.(StepNum>0)) then
            if (MassError > GlobalMassTol) then
               write(LogFile,*) 'Global mass percent error too large:', &
                  MassError,'%'
            end if
            TimeStepFlag = 1
            GlobalMassTol = GlobalMassTol + GlobalMassTol
         end if
      end if
   end if
   if ((Time-LastPrintTime)>=PrintFreq) then
      PrintStep = 1
   else
      PrintStep = 0
   end if
   if (PrintStep==1) then
      open(unit=14,file='MassTrackGlobal.dat',status='old',position='append')
      tInYears = real(Time*(1.D0/3600.D0)*(1.D0/24.D0)*(1.D0/365.25D0))
      write(14,*) '-----------------------------------------------------'
      write(14,*) 'Printing output at time = ',tInYears,' yrs'
      write(14,*) '-----------------------------------------------------'
      write(14,*) 'initial system mass mass (kg): ', InitSystemMass
      write(14,*)
      write(14,*) 'final system mass mass: ', FinlSystemMass
      write(14,*)
      write(14,*) 'total mass into the system: ',TotalMassIn
      write(14,*)
      write(14,*) 'total system mass gain: ',MassGain
      write(14,*)
      write(14,*) 'mass error as percent of total mass: ',MassError,'%'
      close(14)
   end if
end if

end
