function PhaseIndex(Tempr,Presr,Salin)
! Purpose: to decide and store the phase index for each node.  The phases
!          are identified with the following numbers:
!        -100 = error
!          -2 = liquid-halite
!          -1 = liquid
!           0 = liquid-vapor
!           1 = vapor
!           2 = vapor-halite

! Last modified: 5/19/09

use CommonVars, only: Prec,LogFile
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,Salin,Pres3Ph,Pres3Phase,  &
     CritPr,CritiPres,CritSal,CritiSali,Sal3PhL,S3PhaseL,Sal3PhV,        &
     S2PhaseV,Sal2PhL,SoluL,S2PhaseL,SolubilityL,SoluV,SolubilityV,      &
     Sal2PhV
integer :: PhaseIndex

Pres3Ph = Pres3Phase(Tempr)
CritPr  = CritiPres(Tempr)
if (Presr>CritPr) then
   ! supercritical region
   CritSal = CritiSali(Tempr)
   Sal3PhL = S3PhaseL(Tempr)
   if (Salin>=CritSal) then
      if (Salin<=Sal3PhL) then
         ! supercritical liquid
         PhaseIndex = -1
      else
         ! liquid and halite
         PhaseIndex = -2
      end if
   else
     ! supercritical vapor side
     ! we will denote by -1, however
     PhaseIndex = -1
   end if
elseif (Presr>=Pres3Ph) then
   Sal2PhV = S2PhaseV(Tempr,Presr)
   Sal2PhL = S2PhaseL(Tempr,Presr)
   SoluL   = SolubilityL(Tempr)
   if (Salin<Sal2PhV) then
      ! single-phase vapor
      PhaseIndex = 1
   elseif (Salin<=Sal2PhL) then
      ! two-phase region
      PhaseIndex = 0
   elseif (Salin<=SoluL) then
      ! single-phase liquid
      PhaseIndex = -1
   else
      ! co-existing liquid-halite
      PhaseIndex = -2
   end if
elseif (Presr>0.D0) then
   ! sub-three-phase region
   SoluV = SolubilityV(Tempr,Presr)
   if (Salin<=SoluV) then
      ! single-phase vapor
      PhaseIndex = 1
   else
      ! co-existing vapor-halite
      PhaseIndex = 2
   end if
else
   PhaseIndex = -100
end if

return
end
