recursive function Pres3Phase(Tempr) Result(Answer)
! Input: temperature (C)
! Output: pressure (Pa) on the three phase co-existence
!         surface. 

! The 3-phase pressures were arrived at by taking the
! lowest available pressures on the upper boundary of 
! the two-phase region based on Anderko and Pitzer(1993) 
! (see Base2Phase for reference).

use CommonVars, only: TempArray,Pres3PhMat,Prec
implicit none
real(selected_real_kind(Prec)) :: Answer,Tempr,T1,T2,P1,P2,Slope
integer :: Row1,Row2

if (Tempr>=300.D0) then
   call FindTemp(Tempr,Row1,Row2)
   T1 = TempArray(Row1)
   T2 = TempArray(Row2)
   P1 = Pres3PhMat(Row1-30)
   P2 = Pres3PhMat(Row2-30)
   Slope = (P2-P1)/(T2-T1)
   Answer = (Slope*(Tempr-T1)+P1)*1.D5
else
   Answer = Pres3Phase(300.D0)
end if
end
