subroutine PrintOutput(LastPrintTime)
! Purpose: To store output in the files OutFile1,2, and 3.

use commonvars, only: Time,Prec,PhaseIndices,NumNodesX,      &
    NumNodesY,NumNodesZ,BulkDens,Pres,Temp,Sali,LiqVelocX,   &
    LiqVelocY,LiqVelocZ,VapVelocX,VapVelocY,VapVelocZ,       &
    LiqVolSats,VaporSali,LiquidSali,VaporEnth,LiquidEnth,    &
    BulkEnth,VaporDens,LiquidDens,HaliteSat,LogFile
implicit none
integer :: I,J,K
real (selected_real_kind(Prec)) :: tInYears,LastPrintTime,   &
   SurfHeatFluxes(NumNodesX,NumNodesY),FViscosity,           &
   Viscosities(NumNodesX,NumNodesZ)

tInYears = real(Time*(1.D0/3600.D0)*(1.D0/24.D0)*(1.D0/365.25D0))
write(LogFile,*) 'Printing output at time = ',tInYears,' yrs'

!--------------------------------------------------------------------
!                       Primary Output
!--------------------------------------------------------------------
if (LastPrintTime==0.D0) then
   open(unit=14,file='OutFile1',status='replace')
   open(unit=15,file='OutFile2',status='replace')
   open(unit=16,file='OutFile3',status='replace')
   open(unit=17,file='OutFile4',status='replace')
else
   open(unit=14,file='OutFile1',status='old',position='append')
   open(unit=15,file='OutFile2',status='old',position='append')
   open(unit=16,file='OutFile3',status='old',position='append')
   open(unit=17,file='OutFile4',status='old',position='append')
end if

10 format(2000I3)    ! Phase Indices
15 format(2000E14.7) ! Liquid Volume Saturations,Densities
20 format(2000E15.8) ! Pressures
30 format(2000E14.7) ! Temperatures,Densities
40 format(2000E12.5) ! Salinities
50 format(2000E15.8) ! Enthalpies,Velocities

write(14,5) 'Printing output at time = ',tInYears,' yrs'
5 format (A26,D20.15,A4)
write(14,*) '                        = ',real(Time),' secs'
write(14,*)

write(14,*) '--- Phases ---'
write(14,*) '(-2=liquid+halite, -1=liquid, 0=two-phase,'   
write(14,*) '  1=vapor, 2=vapor+halite)'
write(14,*)
do J=1,NumNodesY
   write(14,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(14,10) (PhaseIndices(I,J,K),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(14,*)
   end if
end do

write(14,*)
write(14,*) '--- Liquid Volume Saturation ---'
write(14,*)
do J=1,NumNodesY
   write(14,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(14,15) (real(LiqVolSats(I,J,K)),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(14,*)
   end if
end do

write(14,*)
write(14,*) '--- Densities (kg/m^3) ---'
write(14,*)
do J=1,NumNodesY
   write(14,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(14,15) (real(BulkDens(I,J,K)),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(14,*)
   end if
end do

write(14,*)
write(14,*) '--- Pressures (Pa) ---'
write(14,*)
do J=1,NumNodesY
   write(14,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(14,20) (real(Pres(I,J,K)),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(14,*)
   end if
end do

write(14,*)
write(14,*) '--- Temperatures (C) ---'
write(14,*)
do J=1,NumNodesY
   write(14,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(14,30) (real(Temp(I,J,K)),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(14,*)
   end if
end do

write(14,*)
write(14,*) '--- Salinities (wt% NaCl) ---'
write(14,*)
do J=1,NumNodesY
   write(14,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(14,40) (real(Sali(I,J,K)*1.D2),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(14,*)
   end if
end do

write(14,*)
write(14,*) '--- Bulk Enthalpies (J/kg) ---'
write(14,*)
do J=1,NumNodesY
   write(14,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(14,50) (real(BulkEnth(I,J,K)),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(14,*)
   end if
end do
!--------------------------------------------------------------------
!                       Secondary Output
!--------------------------------------------------------------------
write(15,*) 'Printing output at time = ',tInYears,' yrs'
write(15,*) '                        = ',Time,' secs'
write(15,*)

write(15,*) '--- X Direction Liquid Velocities (m/s) ---'
write(15,*)
if ((NumNodesX-1).ne.0) then
   do I=1,NumNodesX-1
      write(15,*) 'X Direction Interface Slice: ',I
      do K=1,NumNodesZ
         write(15,50) (real(LiqVelocX(I,J,K)),J=1,NumNodesY)
      end do
      if (I<(NumNodesX-1)) then
         write(15,*)
      end if
   end do
else
   write(15,*) 'Does not apply'
end if

write(15,*)
write(15,*) '--- X Direction Vapor Velocities (m/s) ---'
write(15,*)
if ((NumNodesX-1).ne.0) then
   do I=1,NumNodesX-1
      write(15,*) 'X Direction Interface Slice: ',I
      do K=1,NumNodesZ
         write(15,50) (real(VapVelocX(I,J,K)),J=1,NumNodesY)
      end do
      if (I<(NumNodesX-1)) then
         write(15,*)
      end if
   end do
else
   write(15,*) 'Does not apply'
end if

write(15,*)
write(15,*) '--- Y Direction Liquid Velocities (m/s) ---'
write(15,*)
if ((NumNodesY-1).ne.0) then
   do J=1,NumNodesY-1
      write(15,*) 'Y Direction Interface Slice: ',J
      do K=1,NumNodesZ
         write(15,50) (real(LiqVelocY(I,J,K)),I=1,NumNodesX)
      end do
      if (J<NumNodesY-1) then
         write(15,*)
      end if
   end do
else
   write(15,*) 'Does not apply'
end if

write(15,*)
write(15,*) '--- Y Direction Vapor Velocities (m/s) ---'
write(15,*)
if ((NumNodesY-1).ne.0) then
   do J=1,NumNodesY-1
      write(15,*) 'Y Direction Interface Slice: ',J
      do K=1,NumNodesZ
         write(15,50) (real(VapVelocY(I,J,K)),I=1,NumNodesX)
      end do
      if (J<NumNodesY-1) then
         write(15,*)
      end if
   end do
else
   write(15,*) 'Does not apply'
end if

write(15,*)
write(15,*) '--- Z Direction Liquid Velocities (m/s) ---'
write(15,*)
if ((NumNodesZ-1).ne.0) then
   do K=1,NumNodesZ-1
      write(15,*) 'Z Direction Interface Slice: ',K
      do I=1,NumNodesX
         write(15,50) (real(LiqVelocZ(I,J,K)),J=1,NumNodesY)
      end do
      if (K<NumNodesZ-1) then
         write(15,*)
      end if
   end do
else
   write(15,*) 'Does not apply'
end if

write(15,*)
write(15,*) '--- Z Direction Vapor Velocities (m/s) ---'
write(15,*)
if ((NumNodesZ-1).ne.0) then
   do K=1,NumNodesZ-1
      write(15,*) 'Z Direction Interface Slice: ',K
      do I=1,NumNodesX
         write(15,50) (real(VapVelocZ(I,J,K)),J=1,NumNodesY)
      end do
      if (K<NumNodesZ-1) then
         write(15,*)
      end if
   end do
else
   write(15,*) 'Does not apply'
end if
write(15,*)
write(15,*)
!--------------------------------------------------------------------
!                       Tertiary Output
!--------------------------------------------------------------------
write(16,*) 'Printing output at time = ',tInYears,' yrs'
write(16,*) '                        = ',real(Time),' secs'
write(16,*)

write(16,*) '--- Vapor Salinities (wt% NaCl) ---'
write(16,*)
do J=1,NumNodesY
   write(16,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(16,40) (VaporSali(I,J,K)*1.D2,I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(16,*)
   end if
end do

write(16,*)
write(16,*) '--- Liquid Salinities (wt% NaCl) ---'
write(16,*)
do J=1,NumNodesY
   write(16,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(16,40) (LiquidSali(I,J,K)*1.D2,I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(16,*)
   end if
end do

write(16,*)
write(16,*) '--- Vapor Enthalpies (J/kg) ---'
write(16,*) 
do J=1,NumNodesY
   write(16,*) 'Slice: ',J
   do K=1,NumNodesZ 
      write(16,50) (VaporEnth(I,J,K),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(16,*)
   end if
end do

write(16,*) 
write(16,*) '--- Liquid Enthalpies (J/kg) ---'
write(16,*)
do J=1,NumNodesY
   write(16,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(16,50) (LiquidEnth(I,J,K),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(16,*)
   end if
end do

write(16,*) 
write(16,*) '--- Vapor Densities (kg/m^3) ---'
write(16,*) 
do J=1,NumNodesY
   write(16,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(16,30) (VaporDens(I,J,K),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(16,*)
   end if
end do

write(16,*)
write(16,*) '--- Liquid Densities (kg/m^3) ---'
write(16,*)
do J=1,NumNodesY
   write(16,*) 'Slice: ',J
   do K=1,NumNodesZ
      write(16,30) (LiquidDens(I,J,K),I=1,NumNodesX)
   end do
   if (J<NumNodesY) then
      write(16,*)
   end if
end do
!--------------------------------------------------------------------
!                       Quaternary Output
!--------------------------------------------------------------------
call SurfHeatFlux(SurfHeatFluxes)
write(17,*) 'Printing output at time = ',tInYears,' yrs'
write(17,*) '                        = ',real(Time),' secs'
write(17,*)
write(17,*) '--- Surface Heat Fluxes (W / m^2) ---'
write(17,*)
if (NumNodesY==1) then
   do I=1,NumNodesX
      write(17,*) SurfHeatFluxes(I,1)
   end do
else
   ! To be filled in when the code is upgraded to 3D
end if
write(17,*)
write(17,*) '--- Liquid Dynamic Viscosities (Pa s) ---'
write(17,*)
if (NumNodesY==1) then
   do K=1,NumNodesZ
     do I=1,NumNodesX
        Viscosities(I,K) = FViscosity(Temp(I,1,K),Pres(I,1,K), &
                           LiquidSali(I,1,K))
     end do
     write(17,*) (Viscosities(I,K),I=1,NumNodesX)
   end do
else
   ! To be filled in when the code is upgraded to 3D
end if

write(17,*)
write(17,*) '--- Vapor Dynamic Viscosities (Pa s) ---'
write(17,*)
if (NumNodesY==1) then
   do K=1,NumNodesZ
      do I=1,NumNodesX
         Viscosities(I,K) = FViscosity(Temp(I,1,K),Pres(I,1,K), &
                            VaporSali(I,1,K))
      end do
      write(17,*) (Viscosities(I,K),I=1,NumNodesX)
   end do
else
   ! To be filled in when the code is upgraded to 3D
end if

write(17,*)
write(17,*) '--- Halite Volume Saturations ---'
write(17,*)
if (NumNodesY==1) then
   do K=1,NumNodesZ
     write(17,*) (HaliteSat(I,1,K),I=1,NumNodesX)   
   end do
else
   ! To be filled in when the code is upgraded to 3D
end if
write(17,*)
!--------------------------------------------------------------------
close(14)
close(15)
close(16)
close(17)
end
