subroutine ReadEOSVals
! Purpose: reads bulk quantities and phase space boundary values 
! from the data files into shared arrays for the equation of state 
! functions to use. Note: for qualitative information regarding the 
! shapes and relative locations of the phase space boundaries, see 
! Palliser and Mckibbin (1998a) - all references mentioned in this 
! code will be listed after the end of the module in which they 
! are cited.

use CommonVars, only: CritiPMat,VapDens2Ph,LiqDens2Ph,LiqSali2Ph, &
   VapSali2Ph,PresMat2Ph,TempArray,Prec,Pres3PhMat,PresArray,     &
   SaliArray,DensSlices,EnthSlices
implicit none
real (selected_real_kind(Prec)) :: Tempr, CritPresr,LDens2Ph(75), &
   VDens2Ph(75),LSali2Ph(75),VSali2Ph(75),Pres2Ph(75),            &
   DSliceArray(100),HSliceArray(100)
integer :: I,J,K
!---------------------------------------------------------------------
!	Tables for quantities defining the two-phase region
!---------------------------------------------------------------------
open(21,file='CritiPres.dat',status='old')
open(22,file='Pres3Ph.dat',status='old')
open(23,file='LiqSali2Ph.dat',status='old')
open(24,file='VapSali2Ph.dat',status='old')
open(25,file='LiqDens2Ph.dat',status='old')
open(26,file='VapDens2Ph.dat',status='old')
open(27,file='Pres2Ph.dat',status='old')
open(28,file='TempArray.dat',status='old')
do I=1,49
   read(21,*) CritiPMat(I)
   read(22,*) Pres3PhMat(I)
   read(23,*) LSali2Ph
   read(24,*) VSali2Ph
   read(25,*) LDens2Ph
   read(26,*) VDens2Ph
   read(27,*) Pres2Ph
   do J=1,75
      LiqSali2Ph(I,J)=LSali2Ph(J)
      VapSali2Ph(I,J)=VSali2Ph(J)
      LiqDens2Ph(I,J)=LDens2Ph(J)
      VapDens2Ph(I,J)=VDens2Ph(J)
      PresMat2Ph(I,J)=Pres2Ph(J)
   end do
end do
do I=1,79
   read(28,*) TempArray(I)
end do
close(21)
close(22)
close(23)
close(24)
close(25)
close(26)
close(27)
close(28)
!---------------------------------------------------------------------
!       Tables for the bulk quantities
!---------------------------------------------------------------------
open(unit=20,file='PresArray.dat',status='old')
open(unit=21,file='SaliArray.dat',status='old')
open(unit=22,file='DSlices.dat',status='old')
open(unit=23,file='HSlices.dat',status='old')
do I=1,79
   do J=1,200
      read(22,*) DSliceArray
      read(23,*) HSliceArray
      do K=1,100
         DensSlices(I,J,K) = DSliceArray(K)
         EnthSlices(I,J,K) = HSliceArray(K)
      end do
   end do
   read(22,*)
   read(23,*)
end do
do I=1,200
   read(20,*) PresArray(I)
end do
do I=1,100
   read(21,*) SaliArray(I)
end do
close(20)
close(21)
close(22)
close(23)
!---------------------------------------------------------------------
end

! Palliser CP, Mckibbin R (1998) a model for deep geothermal brines, 
!     I:T-p-X state-space description,Transport in Porous Media,33,
!     65-80.
