function S2PhaseV(Tempr,Presr)
! Purpose: To calculate the salinity on the vapor side of the
!          upper boundary of the two-phase region, for a given
!          temperature (C) and pressure (Pa)
! Input: Temperature (C) and pressure (Pa)
! Output: Vapor salinity (wt.fraction NaCl) on the vapor side of 
!         the upper boundary of the two-phase region

use CommonVars, only:Prec
implicit none
real (selected_real_kind(Prec)) :: Tempr,Presr,S2PhaseV,CritiPres, &
   CritPresr,Pres3Phase,Pres3Ph

CritPresr = CritiPres(Tempr)
Pres3Ph   = Pres3Phase(Tempr)
if (Presr > CritPresr) then
   call Base2Phase(Tempr,CritPresr,'D',S2PhaseV)
else if (Presr < Pres3Ph) then
   call Base2Phase(Tempr,Pres3Ph,'D',S2PhaseV)
else
   call Base2Phase(Tempr,Presr,'D',S2PhaseV)
end if

return
end
!*********************************************************************
function S2PhaseL(Tempr,Presr)
! Purpose: To calculate the salinity on the liquid side of the
!          upper boundary of the two-phase region, for a given
!          temperature (C) and pressure (Pa)
! Input: Temperature (C) and pressure (Pa)
! Output: Liquid salinity (wt.fraction NaCl) on the vapor side of
!         the upper boundary of the two-phase region

use CommonVars, only:Prec
implicit none
real (selected_real_kind(Prec)) :: Tempr,Presr,S2PhaseL,CritiPres, &
   CritPresr,Pres3Phase,Pres3Ph

CritPresr = CritiPres(Tempr)
Pres3Ph = Pres3Phase(Tempr)
if (Presr > CritPresr) then
   call Base2Phase(Tempr,CritPresr,'C',S2PhaseL)
else if (Presr < Pres3Ph) then
   call Base2Phase(Tempr,Pres3Ph,'C',S2PhaseL)
else
   call Base2Phase(Tempr,Presr,'C',S2PhaseL)
end if

return
end 
