subroutine SaltTrackGlobal(InitVars)

use CommonVars, only: NumNodesZ,ZInterfSaltFlux,SaltInTop,           &
   InitSystemSalt,FinlSystemSalt,ZNodeWidths,XNodeWidths,            &
   YNodeWidths,BulkDens,Sali,Delta_t,Prec,Porosities,Time,SaltInBot, &
   SaltInWest,SaltInEast,NumNodesX,XInterfSaltFlux,TimeStepCheck,    &
   TimeStepFlag,NumTimeCuts,MaxTimeCuts,LogFile,GlobalSaltTol,       &
   StepNum,Pres,Temp,LastPrintTime,PrintFreq,DeltaSaltGlobal,        &
   GlobalSaltError,SaliNodeTypes,SaliBoundVals
implicit none
real(selected_real_kind(Prec)) :: TotalSaltIn,SaltGain,tInYears,     &
   SaltError,Volume,CalcBulkDens
integer :: PrintStep,InitVars,I,K

if (InitVars==1) then
   GlobalSaltTol = 5.D0
   open(unit=14,file='SaltTrackGlobal.dat',status='replace')
   close(14)
   SaltInTop  = 0.D0
   SaltInBot  = 0.D0
   SaltInWest = 0.D0
   SaltInEast = 0.D0
   InitSystemSalt = 0.D0
   do K=1,NumNodesZ
      do I=1,NumNodesX
         if ((SaliNodeTypes(I,1,K).ne.'U').and. &
            (SaliNodeTypes(I,1,K).ne.'C')) then
            Volume = XNodeWidths(I)*YNodeWidths(1)*ZNodeWidths(K)
            InitSystemSalt = InitSystemSalt + &
               BulkDens(I,1,K)*Sali(I,1,K)*Volume*Porosities(I,1,K)
         end if
      end do
   end do
else
   FinlSystemSalt = 0.D0
   do K=1,NumNodesZ
      do I=1,NumNodesX
         if ((SaliNodeTypes(I,1,K).ne.'U').and. &
            (SaliNodeTypes(I,1,K).ne.'C')) then
            FinlSystemSalt = FinlSystemSalt + &
               CalcBulkDens(Temp(I,1,K),Pres(I,1,K),Sali(I,1,K))* &
               Sali(I,1,K)*XNodeWidths(I)*YNodeWidths(1)* &
               ZNodeWidths(K)*Porosities(I,1,K)
         end if
         if (I>1) then
            if ((SaliNodeTypes(I-1,1,K)=='U').or. &
               (SaliNodeTypes(I-1,1,K)=='C')) then
               SaltInWest = SaltInWest + &
                  XInterfSaltFlux(I-1,1,K)*Delta_t* &
                  ZNodeWidths(K)*YNodeWidths(1)
            elseif (SaliNodeTypes(I-1,1,K)=='F') then
               if ((K.ne.1).and.(K.ne.NumNodesZ)) then
                  SaltInWest = SaltInWest + SaliBoundVals(I-1,1,K)* &
                     ZNodeWidths(K)*Delta_t
               end if
            end if
         end if
         if (I<NumNodesX) then
            if ((SaliNodeTypes(I+1,1,K)=='U').or. &
               (SaliNodeTypes(I+1,1,K)=='C')) then
               SaltInEast = SaltInEast - &
                  XInterfSaltFlux(I,1,K)*Delta_t* &
                  ZNodeWidths(K)*YNodeWidths(1)
            elseif (SaliNodeTypes(I+1,1,K)=='F') then
               if ((K.ne.1).and.(K.ne.NumNodesZ)) then
                  SaltInEast = SaltInEast + SaliBoundVals(I+1,1,K)* &
                     ZNodeWidths(K)*Delta_t
               end if
            end if
         end if
         if (K>1) then
            if ((SaliNodeTypes(I,1,K-1)=='U').or. &
               (SaliNodeTypes(I,1,K-1)=='C')) then
               SaltInTop = SaltInTop + &
                  ZInterfSaltFlux(I,1,K-1)*Delta_t* &
                  XNodeWidths(I)*YNodeWidths(1)
            elseif (SaliNodeTypes(I,1,K-1)=='F') then
               SaltInTop = SaltInTop + SaliBoundVals(I,1,K-1)* &
                  XNodeWidths(I)*Delta_t
            end if
         end if
         if (K<NumNodesZ) then
            if ((SaliNodeTypes(I,1,K+1)=='U').or. &
               (SaliNodeTypes(I,1,K+1)=='C')) then
               SaltInBot = SaltInBot - &
                  ZInterfSaltFlux(I,1,K)*Delta_t*   &
                  XNodeWidths(I)*YNodeWidths(1)
            elseif (SaliNodeTypes(I,1,K+1)=='F') then
               SaltInBot = SaltInBot + SaliBoundVals(I,1,K+1)* &
                  XNodeWidths(I)*Delta_t
            end if
         end if
      end do
   end do
   TotalSaltIn = SaltInTop + SaltInBot + SaltInWest + SaltInEast
   SaltGain = FinlSystemSalt - InitSystemSalt
   SaltError = DABS(SaltGain-TotalSaltIn)
   DeltaSaltGlobal = TotalSaltIn-SaltGain
   SaltError = 100.D0*SaltError/InitSystemSalt
   GlobalSaltError = SaltError/100.D0

   !write(*,*) 'Global salt error: ',SaltError

   if ((TimeStepCheck == 'Y').and.(TimeStepFlag == 0)) then
      if (SaltError > GlobalSaltTol) then
         if ((NumTimeCuts < MaxTimeCuts).and.(StepNum>0)) then
            if (SaltError > GlobalSaltTol) then
               write(LogFile,*) 'Global salt percent error too large:', &
                  SaltError,'%'
            end if
            TimeStepFlag = 1
            GlobalSaltTol = GlobalSaltTol + GlobalSaltTol
         end if
      end if
   end if
   if ((Time-LastPrintTime)>=PrintFreq) then
      PrintStep = 1
   else
      PrintStep = 0
   end if
   if (PrintStep==1) then
      open(unit=14,file='SaltTrackGlobal.dat',status='old',position='append')
      tInYears = real(Time*(1.D0/3600.D0)*(1.D0/24.D0)*(1.D0/365.25D0))
      write(14,*) '-----------------------------------------------------'
      write(14,*) 'Printing output at time = ',tInYears,' yrs'
      write(14,*) '-----------------------------------------------------'
      write(14,*) 'initial system salt mass (kg): ', InitSystemSalt
      write(14,*)
      write(14,*) 'final system salt mass: ', FinlSystemSalt
      write(14,*)
      write(14,*) 'total salt into the system: ',TotalSaltIn
      write(14,*)
      write(14,*) 'total system salt gain: ',SaltGain
      write(14,*)
      write(14,*) 'salt error as percent of total salt: ',SaltError,'%'
      close(14)
   end if
end if

end
