function SolubilityV(tempr,presr)
! Purpose: To calculate the vapor salinity on the surface that
!          separates the vapor region and the vapor-halite region,
!          underneath the 3-phase surface.
! Input: Temperature (C) and pressure (Pa).
! Output: Salt-saturated vapor salinity (wt.fraction NaCl)

use CommonVars, only: Prec,LogFile
implicit none
real(selected_real_kind(Prec)) :: Tempr,Presr,SolubilityV,y, &
   Pres3Phase,S3PhaseV

if (Tempr<=350.D0) then
   y = 6.5D0
elseif (Tempr<800.D0) then
   y = 6.D0-(Tempr-400.D0)/100.D0
else
   y = 0.D0
end if

SolubilityV = S3PhaseV(Tempr)*(Presr/Pres3Phase(Tempr))**y

return
end
!---------------------------------------------------------------------
function SolubilityL(tempr)
! Purpose: to calculate the liquid salinity at the saturated
!          liquid curve-Xlsol, the intersection of regions 1 
!          and 6

use CommonVars, only: Prec
implicit none
real(selected_real_kind(Prec)) :: tempr,SolubilityL,S3PhaseL

if (tempr.LT.100) then
   SolubilityL = 0.D0
   return
end if

solubilityl = s3phasel(tempr)
!**********************************************************************
return
end
