subroutine StaticPres
! calculates the pressure profile that would correspond to a 
! system with no fluid flow, i.e., the static pressure distribution
! N.B., use only when gravity is assumed to be parallel to the Z
! direction

use CommonVars, only: Prec,Pres,NumNodesX,NumNodesY,NumNodesZ,       &
   XNodeWidths,YNodeWidths,ZNodeWidths,Porosities,Grav,Temp,Sali,    &
   LogFile
implicit none
real(selected_real_kind(Prec)) :: DistPN,CalcBulkDens,Dens,PresGuess
integer :: I,J,K,L,NumIter

NumIter = 10
write(LogFile,*) 'Calculating static pressure distribution...'
do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         if (K==1) then
            Dens = CalcBulkDens(Temp(I,J,K),Pres(I,J,K),Sali(I,J,K))
            if (Dens==-1.D0) then
               write(LogFile,*) 'CalcBulkDens returned an error -StaticPres'
               write(LogFile,*) 'Halting'
               stop
            end if
         else
            DistPN = ZNodeWidths(K-1)/2.D0 + ZNodeWidths(K)/2.D0
            do L=1,NumIter
               PresGuess = Pres(I,J,K-1) + &
                           Dens*Grav*DistPN
               Dens = CalcBulkDens(Temp(I,J,K),PresGuess,Sali(I,J,K))
            end do
            Pres(I,J,K) = PresGuess
         end if
         !write(LogFile,*) Depth,I,K
         !write(LogFile,*) Pres(I,J,K)/1.D5,I,K
      end do
   end do
end do
write(LogFile,*) 'Pressures calculated.'
write(LogFile,*)

end
