subroutine SurfHeatFlux(SurfHeatFluxes)
! Purpose: to calculate the surface heat fluxes for printing
!          into output file OutFile4
! SurfHeatFluxes is an output variable that is populated with
! the vertical surface heat fluxes by the time this module
! terminates.

use CommonVars, only: Prec,NumNodesX,NumNodesY,Temp,             &
   FluidThermCond,RockThermCond,Porosities,ZInterfLiqDens,       &
   ZInterfVapDens,ZInterfLiqEnth,ZInterfVapEnth,LiqVelocZ,       &
   VapVelocZ,ZNodeWidths,Temp,Pres
implicit none
real(selected_real_kind(Prec)) :: SurfHeatFluxes(NumNodesX,      &
   NumNodesY),KMedium,DTDZ,ConductFlux,LiqConvectFlux,           &
   VapConvectFlux,NodeDist,RefTemp,RefEnth,RefSali,RefPres,      &
   CalcBulkEnth,MinSurfTemp
integer :: I,J,K

! Note: Z is positive downward, so that negative heat fluxes
! represent heat transport toward the seafloor.
K = 1
RefSali = 3.2D-2
RefPres = Pres(1,1,1)
MinSurfTemp = 1000.D0
do I=1,NumNodesX
   if (Temp(I,1,1)<MinSurfTemp) then
      MinSurfTemp = Temp(I,1,1)
   end if 
end do 
RefTemp = MinSurfTemp
RefEnth = CalcBulkEnth(RefTemp,RefPres,RefSali)
do I=1,NumNodesX
   do J=1,NumNodesY
      KMedium=Porosities(I,J,K)*FluidThermCond +                 &
         (1.D0-Porosities(I,J,K))*RockThermCond
      NodeDist=(ZNodeWidths(K+1)+ZNodeWidths(K))/2.D0
      DTDZ=(Temp(I,J,K+1)-Temp(I,J,K))/NodeDist
      ConductFlux = -KMedium*DTDZ
      LiqConvectFlux = ZInterfLiqDens(I,J,K)*                    &
                       (ZInterfLiqEnth(I,J,K)-RefEnth)*          &
                       LiqVelocZ(I,J,K)
      VapConvectFlux = ZInterfVapDens(I,J,K)*                    &
                       (ZInterfVapEnth(I,J,K)-RefEnth)*          &
                       VapVelocZ(I,J,K)
      ! Eliminate convective component into the crust so that 
      ! the end results will all be negative; they can be 
      ! converted to positive and plotted on a semilog scale.
      if (LiqConvectFlux>0.D0) then
         LiqConvectFlux=0.D0
      end if
      if (VapConvectFlux>0.D0) then
         VapConvectFlux=0.D0
      end if

      SurfHeatFluxes(I,J) = ConductFlux + LiqConvectFlux +       &
                            VapConvectFlux
   end do
end do

end
