subroutine ThermoProps
! Purpose: To calculate the values of bulk and individual phase
!          quantities at every node. These include the "zeta" terms
!          that are in the source of the salinity equation

use CommonVars, only:Prec,PhaseIndices,NumNodesX,NumNodesY,NumNodesZ, &
   Temp,Pres,Sali,BulkDens,BulkEnth,LiquidEnth,VaporEnth,LiquidDens,  &
   VaporDens,LiquidSali,VaporSali,LiqVolSats,VapVolSats,HaliteSat,    &
   HaliteDens,Porosities,OldPoros,LogFile,DensSlices,MinPres,         &
   CompressFactor
              
implicit none
real (selected_real_kind(Prec)) :: S3PhaseV,Sal3PhV,Term1,Term2,      &
   CalcBulkDens,CalcBulkEnth,Sal3PhL,S3PhaseL,S2PhaseL,S2PhaseV,      &
   D2PhaseL,D2PhaseV,H2PhaseL,H2PhaseV,LiqMassFrac,VapMassFrac,       &
   Den3PhL,Pres3Phase,Den3PhV,SolubilityV,Interp3D,RefPres,Alpha,     &
   Beta,HalRefDens,HalDens,RefTemp,Weight1,Weight2,DeltaRho
integer :: I,J,K

do I=1,NumNodesX
   do J=1,NumNodesY
      do K=1,NumNodesZ
         BulkDens(I,J,K)=CalcBulkDens(Temp(I,J,K),Pres(I,J,K), &
                                         Sali(I,J,K))
         if (BulkDens(I,J,K)==-1.D0) then
            write(LogFile,*) 'CalcBulkDens returned an error -ThermoProps'
            write(LogFile,*) 'T: ',Temp(I,J,K)
            write(LogFile,*) 'P: ',Pres(I,J,K)
            write(LogFile,*) 'X: ',Sali(I,J,K)
            write(LogFile,*) 'Halting'
            stop
         end if
         BulkEnth(I,J,K)=CalcBulkEnth(Temp(I,J,K),Pres(I,J,K), &
                                      Sali(I,J,K))
         if (PhaseIndices(I,J,K)==2) then
            ! co-existing vapor-halite
            Sal3PhV = SolubilityV(Temp(I,J,K))
            Den3PhV = Interp3D(Temp(I,J,K),Pres(I,J,K),Sal3PhV, &
               DensSlices)
            Alpha = 1.D-4
            Beta  = 1.D-1
            RefPres = Pres3Phase(Temp(I,J,K))
            RefTemp = 0.D0
            HalRefDens = HaliteDens
            HalDens = HalRefDens+(Alpha*(Pres(I,J,K)-RefPres)- &
            Beta*(Temp(I,J,K)-RefTemp))
            Term1 = Den3PhV*(Sali(I,J,K)-Sal3PhV)
            Term2 = HalDens*(1.D0-Sali(I,J,K))
            HaliteSat(I,J,K) = Term1/(Term1+Term2)
            LiqVolSats(I,J,K) = 0.D0
            VapVolSats(I,J,K) = 1.D0-HaliteSat(I,J,K)
            LiquidEnth(I,J,K) = 0.D0
            VaporEnth(I,J,K) = BulkEnth(I,J,K)
            LiquidSali(I,J,K) = 0.D0
            VaporSali(I,J,K) = Sal3PhV
            LiquidDens(I,J,K) = 0.D0
            VaporDens(I,J,K) = Den3PhV
         elseif (PhaseIndices(I,J,K)==-2) then
            ! co-existing liquid-halite
            Sal3PhL = S3PhaseL(Temp(I,J,K))
            Den3PhL = Interp3D(Temp(I,J,K),Pres(I,J,K),Sal3PhL, &
               DensSlices)
            Alpha = 1.D-4
            Beta  = 1.D-1
            RefPres = Pres3Phase(Temp(I,J,K))
            RefTemp = 0.D0
            HalRefDens = HaliteDens
            HalDens = HalRefDens+(Alpha*(Pres(I,J,K)-RefPres)- &
               Beta*(Temp(I,J,K)-RefTemp))
            Term1 = Den3PhL*(Sali(I,J,K)-Sal3PhL)
            Term2 = HalDens*(1.D0-Sali(I,J,K))
            HaliteSat(I,J,K) = Term1/(Term1+Term2)
            LiqVolSats(I,J,K) = 1.D0-HaliteSat(I,J,K)
            VapVolSats(I,J,K) = 0.D0
            LiquidEnth(I,J,K) = BulkEnth(I,J,K)
            VaporEnth(I,J,K) = 0.D0
            LiquidSali(I,J,K) = Sal3PhL
            VaporSali(I,J,K) = 0.D0
            LiquidDens(I,J,K) = Den3PhL
            VaporDens(I,J,K) = 0.D0
         elseif (PhaseIndices(I,J,K)==1) then
            ! vapor
            LiquidEnth(I,J,K) = 0.D0
            VaporEnth(I,J,K) = BulkEnth(I,J,K)
            LiquidDens(I,J,K) = 0.D0
            VaporDens(I,J,K) = BulkDens(I,J,K)
            LiquidSali(I,J,K) = 0.D0
            VaporSali(I,J,K) = Sali(I,J,K)
            LiqVolSats(I,J,K) = 0.D0
            VapVolSats(I,J,K) = 1.D0
            HaliteSat(I,J,K) = 0.D0
         elseif (PhaseIndices(I,J,K)==-1) then
            ! liquid (supercritical and subcritical)
            LiquidEnth(I,J,K) = BulkEnth(I,J,K)
            VaporEnth(I,J,K) = 0.D0
            LiquidDens(I,J,K) = BulkDens(I,J,K)
            VaporDens(I,J,K) = 0.D0
            LiquidSali(I,J,K) = Sali(I,J,K)
            VaporSali(I,J,K) = 0.D0
            LiqVolSats(I,J,K) = 1.D0
            VapVolSats(I,J,K) = 0.D0
            HaliteSat(I,J,K)=0.D0
         elseif (PhaseIndices(I,J,K)==0) then
            ! Two-phase mixture
            RefPres = MinPres
            LiquidSali(I,J,K) = S2PhaseL(Temp(I,J,K),Pres(I,J,K))
            VaporSali(I,J,K)  = S2PhaseV(Temp(I,J,K),Pres(I,J,K))
            LiquidDens(I,J,K) = D2PhaseL(Temp(I,J,K),Pres(I,J,K))
            VaporDens(I,J,K)  = D2PhaseV(Temp(I,J,K),Pres(I,J,K))
            LiquidEnth(I,J,K) = H2PhaseL(Temp(I,J,K),Pres(I,J,K))
            VaporEnth(I,J,K)  = H2PhaseV(Temp(I,J,K),Pres(I,J,K))
            if (DABS(LiquidSali(I,J,K)-VaporSali(I,J,K))>0.D0) then
               LiqMassFrac = (Sali(I,J,K)-VaporSali(I,J,K))/ &
                             (LiquidSali(I,J,K)-VaporSali(I,J,K))
            else
               LiqMassFrac = 1.D0
            end if
            VapMassFrac = 1.D0-LiqMassFrac
            LiqVolSats(I,J,K)=VaporDens(I,J,K)*LiqMassFrac/   &
                              (VaporDens(I,J,K)*LiqMassFrac + &
                               LiquidDens(I,J,K)*VapMassFrac)
            VapVolSats(I,J,K)=1.D0-LiqVolSats(I,J,K)
            HaliteSat(I,J,K) = 0.D0

            DeltaRho = CompressFactor*(Pres(I,J,K)-RefPres)
            Weight1 = LiqVolSats(I,J,K)**2.D0
            Weight2 = 1.D0-Weight1
            LiquidDens(I,J,K) = LiquidDens(I,J,K)+ &
               (Weight1/LiqVolSats(I,J,K))*DeltaRho
            VaporDens(I,J,K) = VaporDens(I,J,K)+ &
               (Weight2/VapVolSats(I,J,K))*DeltaRho
         else
            write(LogFile,*) 'No Phase Index for this Node! - ThermoProps'
            write(LogFile,*) 'Coord: ',I,J,K
            write(LogFile,*) 'Phase Index: ',PhaseIndices(I,J,K)
            write(LogFile,*) 'Code terminating.'
            STOP
         end if
         ! The two lines below are only for the Elder problem 
         ! simulation!! See also the lines at the end of
         ! CalcBulkDens and in GetVelocities
         !BulkDens(I,J,K) = 1000.D0 + 200.D0*Sali(I,J,K)
         !HaliteSat(I,J,K)= 0.D0
      end do
   end do
end do
end
