subroutine TriDiMatSolv(LowerDiag,MiddleDiag,UpperDiag,RHSConstants, &
                        Solutions)
! Purpose: To find the solution to a set of linear equations that 
!          form a tridiagonal matrix.
! Input: Terms from the lower diagonal (just below the diagonal of the
!        matrix), terms from the diagonal, and terms from the upper
!        diagonal above it; RHSConstants are the constants on the
!        right-hand-side of the set of linear equations.
! Output: An array containing the solution to the system of 
!         equations.

! Note: all the extra entries in the parameters above should be set to
!       -100.D0, and also don't put negatives on the off diagonal 
!       terms or the RHS constants

use CommonVars, only: Prec, MaxLongArray
implicit none
real (selected_real_kind(Prec)) :: LowerDiag(MaxLongArray),        &
     MiddleDiag(MaxLongArray),UpperDiag(MaxLongArray),             &
     RHSConstants(MaxLongArray),P(MaxLongArray),Q(MaxLongArray),   &
     Solutions(MaxLongArray),Upper,Middle,Lower,PTerm,RHSTerm,     &
     QTerm
integer :: N,I

! Get P1 and Q1
P(1) = UpperDiag(1)/MiddleDiag(1)
Q(1) = RHSConstants(1)/MiddleDiag(1)

N=2
do while (MiddleDiag(N).ne.-100.D0)
   N=N+1
end do

! Get the rest of the Ps and Qs
do I=2,N-1
   Upper = UpperDiag(I)
   Middle = MiddleDiag(I)
   Lower = LowerDiag(I)
   PTerm = P(I-1)
   QTerm = Q(I-1)
   RHSTerm = RHSConstants(I)
   P(I)= Upper/(Middle-Lower*PTerm)
   Q(I)= (RHSTerm+Lower*QTerm)/(Middle-    &
          Lower*PTerm)
end do
I = N-1

! Set Solutions(N) = Q(N) where N is the last one
Solutions(I)=Q(I)
I = I - 1

do while (I>=1)
   Solutions(I)=P(I)*Solutions(I+1)+Q(I)
   I = I - 1
end do
end
