module CommonVars
! REMINDER: get rid of all the unnecessary Y variables!!!

! NOTE: modules should only be allowed access to the variables from 
!       CommonVars that they might actually need to use.

! IMPORTANT: MaxX,MaxY, and MaxZ are the maximum number of nodes allowable
!    in the X,Y, and Z directions. If they are set too high, then all the 
!    RAM memory space will be used up. Also, if one tries to run FISHES 
!    with, say, 100 nodes in the Z direction while MaxZ is only set to, 
!    say, 90, then the code will not work properly, and there may not be 
!    any error messages that indicate this particular problem! Hence, be 
!    wary that these variables are set to appropriate values.
! NOTE ALSO: MaxLongArray should be as long as the largest of MaxX,
!            MaxY, and MaxZ
integer, parameter :: Prec = 15, MaxX = 160, MaxY = 1, MaxZ = 200, &
   MaxLongArray = 200,MaxShortArray = 20,MaxTimeCuts = 4,          &
   Unity = 1,Zero = 0

! MaxTemp/Pres/SaliChange is the maximum amount these quantities are
! allowed to change during a single time step
real(selected_real_kind(Prec)), parameter :: Pi = &
   3.1415926535897932384626433832795,MaxTempChange = 10.D0, &
   MaxPresChange = 10.D5,MaxSaliChange = 1.D-1,             &
   MaxPres = 1000.D5,MinPres = 85.D5,MaxTemp = 800.D0,      &
   MinTemp = 0.D0,MinSali = 0.D0,MaxSali = 1.D0

! diagnostic variables for checking global mass, salt, and energy
! conservation 
real(selected_real_kind(Prec)) :: MassInTop,InitSystemMass,  &
   FinlSystemMass,SaltInTop,InitSystemSalt,FinlSystemSalt,   &
   MassInBot,SaltInBot,MassInWest,MassInEast,SaltInWest,     &
   SaltInEast,EnergyInTop,InitSystemEnergy,FinlSystemEnergy, &
   EnergyInBot,EnergyInWest,EnergyInEast,EnergyDiffWest,     &
   EnergyDiffEast,EnergyDiffTop,EnergyDiffBot,GlobalMassTol, &
   GlobalSaltTol,GlobalEnergyTol

! diagnostic variables for checking local mass, salt, and energy
! conservation
real(selected_real_kind(Prec)) :: PrevMass(MaxX,MaxY,MaxZ), &
   PrevSalt(MaxX,MaxY,MaxZ),PrevEnergy(MaxX,MaxY,MaxZ),     &
   TotalSaltIn(MaxX,MaxY,MaxZ),TotalMassIn(MaxX,MaxY,MaxZ), &
   MassErrNum,EnergyErrNum
!--------------------------------------------------------------------
! Variables used in the equation of state subroutines        
!--------------------------------------------------------------------
real (selected_real_kind(Prec)) :: CritiPMat(49),VapDens2Ph(49,75), &
   LiqDens2Ph(49,75),VapSali2Ph(49,75),LiqSali2Ph(49,75)

real(selected_real_kind(Prec)) :: Pres3PhMat(49),PresMat2Ph(49,75), &
   TempArray(79)

real(selected_real_kind(Prec)) :: PresArray(200),SaliArray(100),    &
   DensSlices(79,200,100),EnthSlices(79,200,100)
!--------------------------------------------------------------------
     
real (selected_real_kind(Prec)) :: SaltChemDiff,IncreaseAmount, &
   IncreaseTime

real (selected_real_kind(Prec)) :: XInterfMassFlux(MaxX,MaxY,MaxZ),       &
   ZInterfMassFlux(MaxX,MaxY,MaxZ),XInterfSaltFlux(MaxX,MaxY,MaxZ),       &
   ZInterfSaltFlux(MaxX,MaxY,MaxZ),XInterfHeatFlux(MaxX,MaxY,MaxZ),       &
   ZInterfHeatFlux(MaxX,MaxY,MaxZ)
     
real (selected_real_kind(Prec)) :: OldSali(MaxX,MaxY,MaxZ),               &
     OldTemp(MaxX,MaxY,MaxZ),OldPres(MaxX,MaxY,MaxZ),                     &
     OldDens(MaxX,MaxY,MaxZ),OldEnth(MaxX,MaxY,MaxZ), TempRelax,          &
     PresRelax,InitTempDiff(MaxX,MaxY,MaxZ),InitPresDiff(MaxX,MaxY,MaxZ), &
     InitSaliDiff(MaxX,MaxY,MaxZ),PresGradX(MaxX,MaxY,MaxZ),              &
     PresGradY(MaxX,MaxY,MaxZ),PresGradZ(MaxX,MaxY,MaxZ)

real (selected_real_kind(Prec)) :: SaliRelax,HaliteSat(MaxX,MaxY,MaxZ),   &
   XInterfHaliteSat(MaxX,MaxY,MaxZ),YInterfHaliteSat(MaxX,MaxY,MaxZ),     &
   ZInterfHaliteSat(MaxX,MaxY,MaxZ)

integer :: NumNodesX,NumNodesY,NumNodesZ,NumOf_t_Periods,                 &
     PhaseIndices(MaxX,MaxY,MaxZ),TimeCalibration,LineByLineSwitch,       &
     LogFile,TimeStepFlag,ComputeStaticPres,NumTimeCuts,StartBoundNode,   &
     EndBoundNode,MaxPIter,MaxTIter

character :: PresNodeTypes(MaxX,MaxY,MaxZ),TempNodeTypes(MaxX,MaxY,MaxZ), &
   SaliNodeTypes(MaxX,MaxY,MaxZ),SuppressScreen,TimeStepCheck,            &
   BoundValGradual,BoundSide,EquationType

real (selected_real_kind(Prec)) :: Time,TimeStepIncr,RockDens,CpRock,     &
     RockThermCond,Grav,Tilt,HaliteDens,CpHalite,XNodeWidths(MaxX),       &
     YNodeWidths(MaxY),ZNodeWidths(MaxZ),Porosities(MaxX,MaxY,MaxZ),      &
     RockPerms(MaxX,MaxY,MaxZ),FluidThermCond,OldPoros(MaxX,MaxY,MaxZ)

real (selected_real_kind(Prec)) :: Dispersivity,CorrelationLength

real (selected_real_kind(Prec)) :: Temp(MaxX,MaxY,MaxZ),                  &
   Sali(MaxX,MaxY,MaxZ),Pres(MaxX,MaxY,MaxZ)
     
real (selected_real_kind(Prec)) :: PrintFreqArray(MaxShortArray),         &
     tPeriodLengths(MaxShortArray),Delta_t_Array(MaxShortArray),          &
     PresBoundVals(MaxX,MaxY,MaxZ),TempBoundVals(MaxX,MaxY,MaxZ),         &
     StartBoundVals(MaxLongArray)
     
real (selected_real_kind(Prec)) :: SaliBoundVals(MaxX,MaxY,MaxZ),         &
   Delta_t,TotalTime,LiqVolSats(MaxX,MaxY,MaxZ),                          &
   VapVolSats(MaxX,MaxY,MaxZ),MaxDelta_t_Array(MaxShortArray),            &
   MinDelta_t_Array(MaxShortArray)

real (selected_real_kind(Prec)) :: XInterfPoros(MaxX,MaxY,MaxZ),          &
     YInterfPoros(MaxX,MaxY,MaxZ),ZInterfPoros(MaxX,MaxY,MaxZ),           &
     InterfPermsX(MaxX,MaxY,MaxZ),XInterfLiqVolSats(MaxX,MaxY,MaxZ),      &
     InterfPermsY(MaxX,MaxY,MaxZ),YInterfLiqVolSats(MaxX,MaxY,MaxZ),      &
     InterfPermsZ(MaxX,MaxY,MaxZ),ZInterfLiqVolSats(MaxX,MaxY,MaxZ),      &
     XInterfVapVolSats(MaxX,MaxY,MaxZ),YInterfVapVolSats(MaxX,MaxY,MaxZ), &
     ZInterfVapVolSats(MaxX,MaxY,MaxZ)
     
real (selected_real_kind(Prec)) :: XInterfVapDens(MaxX,MaxY,MaxZ),        &
     XInterfLiqDens(MaxX,MaxY,MaxZ)

real(selected_real_kind(Prec)) :: ZInterfVapDens(MaxX,MaxY,MaxZ),         &
     ZInterfLiqDens(MaxX,MaxY,MaxZ),XInterfVapSali(MaxX,MaxY,MaxZ),       &
     XInterfLiqSali(MaxX,MaxY,MaxZ) 
     
real (selected_real_kind(Prec)) :: ZInterfVapSali(MaxX,MaxY,MaxZ),        &
     ZInterfLiqSali(MaxX,MaxY,MaxZ),PresGamma1X(MaxX,MaxY,MaxZ)

real (selected_real_kind(Prec)) :: PresGamma1Y(MaxX,MaxY,MaxZ),           &
     PresGamma1Z(MaxX,MaxY,MaxZ),PresGamma2Z(MaxX,MaxY,MaxZ),             &
     XInterfPres(MaxX,MaxY,MaxZ),ZInterfPres(MaxX,MaxY,MaxZ),             &
     PresGamma2X(MaxX,MaxY,MaxZ)

real (selected_real_kind(Prec)) :: XInterfVapEnth(MaxX,MaxY,MaxZ),        &
     ZInterfVapEnth(MaxX,MaxY,MaxZ),XInterfLiqEnth(MaxX,MaxY,MaxZ)

real (selected_real_kind(Prec)) :: ZInterfLiqEnth(MaxX,MaxY,MaxZ),        &
     ZInterfTemp(MaxX,MaxY,MaxZ),XInterfTemp(MaxX,MaxY,MaxZ)

! Note: all the velocities are stored at the control volume interfaces
real (selected_real_kind(Prec)) :: VapVelocX(MaxX,MaxY,MaxZ),             &
     VapVelocY(MaxX,MaxY,MaxZ),VapVelocZ(MaxX,MaxY,MaxZ),                 &
     LiqVelocX(MaxX,MaxY,MaxZ),LiqVelocY(MaxX,MaxY,MaxZ)
     
real (selected_real_kind(Prec)) :: LiqVelocZ(MaxX,MaxY,MaxZ),             &
     BulkDens(MaxX,MaxY,MaxZ),VaporDens(MaxX,MaxY,MaxZ),                  &
     LiquidDens(MaxX,MaxY,MaxZ),BulkEnth(MaxX,MaxY,MaxZ),                 &
     VaporEnth(MaxX,MaxY,MaxZ),VaporSali(MaxX,MaxY,MaxZ)

! "Interf" stands for "Interface", below and elsewhere in this code.
real (selected_real_kind(Prec)) :: XInterfLiqSaltGrad(MaxX,MaxY,MaxZ),    &
     ZInterfLiqSaltGrad(MaxX,MaxY,MaxZ),                                  &
     XInterfVapSaltGrad(MaxX,MaxY,MaxZ),                                  &
     ZInterfVapSaltGrad(MaxX,MaxY,MaxZ)

real (selected_real_kind(Prec)) :: LiquidSali(MaxX,MaxY,MaxZ),            &
     LiquidEnth(MaxX,MaxY,MaxZ)

! For the Temperature, pressure, and salinity equations, each node
! will have associated with it 8 values: aP,aN,aS,aW,aE,aB,aT, and the
! control volume averaged source term S. These will be stored in four
! dimensional arrays: the first three indices will refer to the node,
! and the last will refer to which of these things is stored:
! 1=aP, 2=aN, 3=aS, 4=aW, 5=aE, 6=aB, 7=aT, and 8=S
! For the meaning of these coefficients, see Patankar (1980)
real (selected_real_kind(Prec)) :: TempCoeffs(MaxX,MaxY,MaxZ,8),          &
   PresCoeffs(MaxX,MaxY,MaxZ,8),SaliCoeffs(MaxX,MaxY,MaxZ,8)

! Variables to store the density derivatives
real (selected_real_kind(Prec)) :: DRhoDPres(MaxX,MaxY,MaxZ),             &
     DRhoDSali(MaxX,MaxY,MaxZ),DRhoDTemp(MaxX,MaxY,MaxZ)

end module CommonVars
