program MatGen
! Purpose: to generate matrices from the FISHES output files 
! and put them in a form usable by matlab or octave

! The matrices are stored in following files (the file unit
! number used in this program to identify each file is in 
! parentheses to the right:

! Phases.dat (17)  LiqVolSat.dat (18) BulkDens.dat  (19)
! Presr.dat  (20)  Tempr.dat     (21) BulkSaln.dat  (22)
! OutFile1   (15)  ZWidths.dat   (24)

implicit none
integer :: Flag,Counter,XNodes,YNodes,ZNodes,I,J,K,                    &
           Phases(1000,1000),EOF
integer, parameter :: Prec = 15
character(len=3) :: Check, Delimeter
real(selected_real_kind(Prec)) :: LiqVolSats(1000,1000),               &
   BulkDens(1000,1000),                                                &
   Presr(1000,1000),Tempr(1000,1000),BulkSaln(1000,1000),              &
   XWidths(1000),ZWidths(1000),BulkEnth(1000,1000),                    &
   VaporSali(1000,1000),LiquidSali(1000,1000),VaporEnth(1000,1000),    &
   LiquidEnth(1000,1000),LiquidVeloc(1000,1000),VaporVeloc(1000,1000), &
   LiquidDens(1000,1000),VaporDens(1000,1000),ULiqVelComp(1000,1000),  &
   UVapVelComp(1000,1000),VLiqVelComp(1000,1000),                      &
   VVapVelComp(1000,1000),SurfHeatFlux  

Delimeter = '   '

call OpenMatFiles

10 format(1000I3)    ! Phase Indices
15 format(1000E14.7) ! Liquid Volume Saturations,Densities
20 format(1000E15.8) ! Pressures
30 format(1000E14.7) ! Temperatures,Densities
40 format(1000E12.5) ! Salinities
50 format(1000E16.8) ! Enthalpies,Velocities

! Find out from InFile0 what the problem dimensions are
call ReadDimensions(XNodes,YNodes,ZNodes,XWidths,ZWidths)

! Show which times are available in OutFile1, ask the user i
! to select the time they want, and then go to that position
call SelectTime

call SkipLines(15,7)
do K=1,ZNodes
   read(15,*) (Phases(K,I),I=1,XNodes)
   write(17,10) (Phases(K,I),I=1,XNodes)
end do

call SkipLines(15,4)
do K=1,ZNodes
   read(15,*) (LiqVolSats(K,I),I=1,XNodes)
   write(18,15) (LiqVolSats(K,I),I=1,XNodes)
end do

call SkipLines(15,4)
do K=1,ZNodes
   read(15,*) (BulkDens(K,I),I=1,XNodes) 
   write(19,15) (BulkDens(K,I),I=1,XNodes)
end do

call SkipLines(15,4)
do K=1,ZNodes
   read(15,*) (Presr(K,I),I=1,XNodes)
   write(20,20) (Presr(K,I),I=1,XNodes)
end do

call SkipLines(15,4)
do K=1,ZNodes
   read(15,*) (Tempr(K,I),I=1,XNodes)
   write(21,30) (Tempr(K,I),I=1,XNodes)
end do

call SkipLines(15,4)
do K=1,ZNodes
   read(15,*) (BulkSaln(K,I),I=1,XNodes)
   write(22,40) (BulkSaln(K,I),I=1,XNodes)
end do

call SkipLines(15,4)
do K=1,ZNodes
   read(15,*) (BulkEnth(K,I),I=1,XNodes)
   write(25,50) (BulkEnth(K,I),I=1,XNodes)
end do

do K=1,ZNodes
   write(24,*) ZWidths(K)
end do

do I=1,XNodes
   write(23,*) XWidths(I)
end do

call SkipLines(30,6)
do K=1,ZNodes
   read(30,*) (VaporSali(K,I),I=1,XNodes)
   write(26,40) (VaporSali(K,I),I=1,XNodes)
end do

call SkipLines(30,4)
do K=1,ZNodes
   read(30,*) (LiquidSali(K,I),I=1,XNodes)
   write(27,40) (LiquidSali(K,I),I=1,XNodes)
end do

call SkipLines(30,4)
do K=1,ZNodes
   read(30,*) (VaporEnth(K,I),I=1,XNodes)
   write(28,50) (VaporEnth(K,I),I=1,XNodes)
end do

call SkipLines(30,4)
do K=1,ZNodes
   read(30,*) (LiquidEnth(K,I),I=1,XNodes)
   write(29,50) (LiquidEnth(K,I),I=1,XNodes)
end do

call SkipLines(30,4)
do K=1,ZNodes
   read(30,*) (VaporDens(K,I),I=1,XNodes)
   write(34,30) (VaporDens(K,I),I=1,XNodes)
end do

call SkipLines(30,4)
do K=1,ZNodes
   read(30,*) (LiquidDens(K,I),I=1,XNodes)
   write(35,30) (LiquidDens(K,I),I=1,XNodes)
end do

if (XNodes==1) then
!------------------
call SkipLines(31,20)
do K=1,ZNodes-1
   read(31,*)
   read(31,*)
   read(31,*) LiquidVeloc(K,1)
   write(33,*) LiquidVeloc(K,1)
end do

call SkipLines(31,2)
do K=1,ZNodes-1
   read(31,*)
   read(31,*)
   read(31,*) VaporVeloc(K,1)
   write(32,*) VaporVeloc(K,1)
end do
!------------------
else
   call SkipLines(31,6)
   do I=1,XNodes-1
      do K=1,ZNodes
         read(31,*) ULiqVelComp(K,I)
      end do
      read(31,*)
      read(31,*)
   end do
   do K=1,ZNodes
      write(40,50) (ULiqVelComp(K,I),I=1,XNodes-1)
   end do
   call SkipLines(31,2)
   do I=1,XNodes-1
      do K=1,ZNodes
         read(31,*) UVapVelComp(K,I)
      end do
      read(31,*)
      read(31,*)
   end do
   do K=1,ZNodes
      write(41,50) (UVapVelComp(K,I),I=1,XNodes-1)
   end do
   call SkipLines(31,10)
   do I=1,ZNodes-1
      do K=1,XNodes
         read(31,*) VLiqVelComp(K,I)
      end do
      read(31,*)
      read(31,*)
   end do
   do K=1,XNodes
      write(42,50) (VLiqVelComp(K,I),I=1,ZNodes-1)
   end do
   call SkipLines(31,2)
   do I=1,ZNodes-1
      do K=1,XNodes
         read(31,*) VVapVelComp(K,I)
      end do
      read(31,*)
      read(31,*,iostat=EOF)
   end do
   do K=1,XNodes
      write(43,50) (VVapVelComp(K,I),I=1,ZNodes-1)
   end do
end if
call SkipLines(44,5)
if (YNodes==1) then
   do I=1,XNodes
      read(44,*,iostat=EOF) SurfHeatFlux
      write(45,*) SurfHeatFlux
   end do
end if
call CloseMatFiles
end
